/*
 *  Copyright (C) 2002 Steve Harris, ECS, University of Southampton
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  $Id: okbc.c,v 1.4 2004/05/22 13:26:18 theno23 Exp $
 */

/* This file includes the global variables, utilities and not much else */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "rdfsql/rdfsql.h"

#include "okbc.h"

RDFSQL *okbc_intl_db = NULL;

void okbc_dump_node_intl(okbc_node *node, int depth);

char *key_none = NULL;
char *key_taxonomic = NULL;
char *key_all = NULL;
char *key_either = NULL;
char *key_equal = NULL;
char *key_error_if_not_unique = NULL;
char *key_filled = NULL;
char *key_known_true = NULL;
char *key_own = NULL;
char *key_stop = NULL;
char *key_system_default = NULL;

okbc_node *okbc_new_node(int type, void *ptr)
{
	okbc_node *new = malloc(sizeof(okbc_node));

	new->type = type;
	if (type == OKBC_LIST) {
		new->data.list = (okbc_list *)ptr;
	} else {
		new->data.atom = ptr;
	}

	return new;
}

okbc_list *okbc_new_list(int length)
{
	okbc_list *new = malloc(sizeof(okbc_list));
	okbc_node **items = malloc(length * sizeof(okbc_node *));

	new->length = length;
	new->items = items;

	return new;
}

okbc_node *okbc_new_list_node(int length)
{
	return okbc_new_node(OKBC_LIST, okbc_new_list(length));
}

void okbc_dump_node(okbc_node *node)
{
	okbc_dump_node_intl(node, 0);
}

void okbc_dump_node_intl(okbc_node *node, int depth)
{
	int i;

	for (i=0; i<depth; i++) {
		printf("   ");
	}
	if (!node) {
		printf("(null)\n");
		return;
	}
	switch (node->type) {
		case OKBC_LIST:
			printf("list\n");
			for (i = 0; i < node->data.list->length; i++) {
				okbc_dump_node_intl(node->data.list->items[i], depth+1);
			}
			break;
		case OKBC_URI:
			printf("[uri] %s\n", (char *)node->data.atom);
			break;
		case OKBC_LITERAL:
			printf("[literal] %s\n", (char *)node->data.atom);
			break;
		case OKBC_SYMBOL:
			printf("[symbol] %s\n", (char *)node->data.atom);
			break;
		case OKBC_BOOL:
			printf("[bool] %s\n", (int)node->data.atom == 0 ? "false" : "true");
			break;
		default:
			printf("[unknown] %p\n", node->data.atom);
			break;
	}
}

void *okbc_new_atom(char *str)
{
	if (str) {
		return (void *)strdup(str);
	} else {
		return NULL;
	}
}

void okbc_init()
{
	key_none = ":none";
	key_taxonomic = ":taxonomic";
	key_all = ":all";
	key_either = ":either";
	key_equal = ":equal";
	key_error_if_not_unique = ":error-if-not-unique";
	key_filled = ":filled";
	key_known_true = ":known-true";
	key_own = ":own";
	key_stop = ":stop";
	key_system_default = ":system-default";
}
