/*
 *  Copyright (C) 2002 Steve Harris, ECS, University of Southampton
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  $Id: okbc_coerce.c,v 1.2 2003/02/20 19:01:15 theno23 Exp $
 */

#include <stdio.h>

#include "rdfsql/rdfsql.h"
#include "okbc_types.h"

/* okbc_coerce_to_class(thing, kb, error_p, kb_local_only_p)
 *
 * (thing &key kb (error-p true) kb-local-only-p)
 *
 * Return Value(s): class class-found-p 
 * 
 * Flags: O R 
 * 
 * Coerces thing to a class. This operation returns two values.
 * 
 * class - If thing identifies a class for kb,
 * then this value is the class so identified, or
 * falseotherwise.
 * class-found-p - If the class is found then true,
 * otherwise false.
 * If error-p is trueand the class is not found, then a
 * class-not-found error is signaled.
 *
 * It is an error to call coerce-to-class with error-p being
 * true, and with a value of thing that does not uniquely identify
 * a class. If this happens, a not-unique-error error should be
 * signaled.
 *
 * Note that in some KRS, false may be a valid class. No portable
 * program may assume that a returned value of false for the first
 * (class) returned value implies that class-found-p
 * is false.
 */

okbc_node *okbc_coerce_to_class(const char *thing, const char *kb,
				int error_p, int kb_local_only_p)
{
    okbc_node *ret = NULL;

    return ret;
}

/* okbc_coerce_to_facet(thing, kb, error_p, kb_local_only_p)
 *
 * (thing &key kb (error-p true) kb-local-only-p)
 *
 * Return Value(s): facet facet-found-p 
 * 
 * Flags: O R 
 * 
 * Coerces thing to a facet. This operation returns two values.
 * 
 * facet - If thing identifies a facet for kb,
 * then this value is the facet so identified, or
 * falseotherwise.
 * facet-found-p - If the facet is found then true,
 * otherwise false.
 * If error-p is trueand the facet is not found, then a
 * slot-not-found error is signaled.
 *
 * It is an error to call coerce-to-facet with error-p being
 * true, and with a value of thing that does not uniquely identify
 * a facet. If this happens, a not-unique-error error should be
 * signaled.
 *
 * Note that in some KRS, false may be a valid facet. No portable
 * program may assume that a returned value of false for the first
 * (facet) returned value implies that facet-found-p is
 * false.
 */

okbc_node *okbc_coerce_to_facet(const char *thing, const char *kb,
				int error_p, int kb_local_only_p)
{
    okbc_node *ret = NULL;

    return ret;
}

/* okbc_coerce_to_frame(thing, kb, error_p, kb_local_only_p)
 *
 * (thing &key kb (error-p true) kb-local-only-p)
 *
 * Return Value(s): frame frame-found-p 
 * 
 * Flags: M R 
 * 
 * Coerces thing to be a frame object, if such an object exists for
 * the underlying KRS, or a frame handle otherwise. Thing can be a
 * frame object or a frame handle. This operation may be
 * less careful than get-frame-in-kb about ensuring that the
 * frame for thing is actually in kb when the supplied
 * thing is a frame object. Coerce-to-frame verifies that
 * thing is the
 * appropriate type of frame object for kb, but may not
 * actually determine whether the frame resides in kb. Therefore, this
 * operation may be faster than get-frame-in-kb for some KRSs.
 *
 * For user convenience, implementors are encouraged to support the coercion
 * into a frame of any data-structure that uniquely identifies a frame in
 * the KRS as well as frame handles and frame objects. It is not
 * portable to provide any value for thing other than a frame
 * object or frame handle; get-frames-matching should be used instead.
 *
 * If the :frame-names-required behavior has the value true for
 * kb, the names of frames are always coercible to frames.
 * If the :frame-names-required behavior is false, frame names
 * (if defined) are not guaranteed to be coercible to frames.
 *
 * This operation returns two values.
 * 
 * frame - If thing identifies a frame for kb,
 * then this value is the frame so identified, or
 * false otherwise.
 * frame-found-p - If the frame is found then true,
 * otherwise false.
 * If error-p is true and the frame is not found, then a
 * not-coercible-to-frame error is signaled.
 *
 * It is an error to call coerce-to-frame with error-p being
 * true, and with a value of thing that does not uniquely identify
 * a frame. If this happens, a not-unique-error error should be
 * signaled.
 *
 * Note that in some KRS, false may be a valid frame object. No portable
 * program may assume that a returned value of false for the first
 * (frame) returned value implies that frame-found-p is
 * false.
 */

okbc_node *okbc_coerce_to_frame(const char *thing, const char *kb,
				int error_p, int kb_local_only_p)
{
    okbc_node *ret = NULL;

    return ret;
}

/* okbc_coerce_to_individual(thing, kb, error_p, kb_local_only_p)
 *
 * (thing &key kb (error-p true) kb-local-only-p)
 *
 * Return Value(s): individual individual-found-p 
 * 
 * Flags: O R 
 * 
 * Coerces thing to an individual. This operation returns two values.
 * 
 * individual - If thing identifies an individual for
 * kb, then this value is the individual so identified, or
 * falseotherwise.
 * individual-found-p - If the individual is found then true,
 * otherwise false.
 * If error-p is trueand the individual is not found, then a
 * individual-not-found error is signaled.
 *
 * It is an error to call coerce-to-individual with error-p
 * being true, and with a value of thing that does not uniquely
 * identify an individual. If this happens, a not-unique-error error
 * should be signaled.
 *
 * Note that in most KRS, false is a valid individual. No portable
 * program may assume that a returned value of false for the first
 * (individual) returned value implies that individual-found-p
 * is false.
 */

okbc_node *okbc_coerce_to_individual(const char *thing, const char *kb,
				     int error_p, int kb_local_only_p)
{
    okbc_node *ret = NULL;

    return ret;
}

/* okbc_coerce_to_kb_value(string_or_value, target_context, kb, wildcards_allowed_p, force_case_insensitive_p, error_p, frame_action, kb_local_only_p)
 *
 * (string-or-value target-context &key kb wildcards-allowed-p force-case-insensitive-p (error-p true) (frame-action :error-if-not-unique) kb-local-only-p)
 *
 * Return Value(s): result-of-read success-p completions-alist 
 * 
 * Flags: O R 
 * 
 * This operation is called by applications that receive input, often from a
 * user in the form of typed input, or a value. Coerce-to-kb-value
 * takes this input and delivers a value that is meaningful to the KRS. This
 * allows applications to interact with users and prompt for expressions
 * containing frame references in a manner that will work predictably across
 * implementation languages, and in networked implementations.
 * Coerce-to-kb-value implements OKBC's reading model just as
 * value-as-string implements OKBC's printing model.
 *
 * string-or-value may be one of the following.
 * 
 * an arbitrary OKBC value entity - If this is a list, then the
 * coercion process applies recursively to the elements of the list.
 * For example, if in the KB the symbol fred is coercible to the
 * frame #&lt;frame FRED 763736&gt;, the value (a fred 42)
 * would be coerced to the KB value
 * (a #&lt;frame FRED 763736&gt; 42).
 * a string - This must be the printed representation of an OKBC entity,
 * possibly containing wildcards. For example, the string
 * "(a fred 42)" would be coerced to the same KB value as in
 * the example above.
 * 
 * Given a string-or-value and a target-context, returns three
 * values.
 
 * 1.
 * result-of-read - the result of reading from the string or
 * value, interpreting objects in the target-context for the
 * kb
 2.
 * success-p - false if an error occurred during the
 * coercion, and true otherwise
 3.
 * completions-alist - an association list of possible
 * completions
 * The first value returned (result-of-read) will be an entity such
 as a string, number, symbol, list (possibly containing other such values),
 or frame.
 *
 * Target-context is one of {:frame, :class,
 :slot, :individual, :facet, :value} and
 identifies the way in which the value to be extracted from
 string-or-value is to be interpreted.
 
 * :frame - It is to be used as a frame argument to
 an OKBC operation. It will be expected to resolve to a frame.
 * :slot - It is to be used as a slot argument to
 an OKBC operation. It will be expected to resolve to a slot name if slots
 are not represented as frames in kb, or to a slot frame if slots
 are represented as frames.
 * :facet - It is to be used as a facet argument to
 an OKBC operation. It will be expected to resolve to a facet name if facets
 are not represented as frames in kb, or to a facet frame if facets
 are represented as frames.
 * :class - It is to be used as a class argument to
 an OKBC operation. It will be expected to resolve to a class name if classes
 are not represented as frames in kb, or to a class frame if classes
 are represented as frames.
 * :individual - It is to be used as an individual
 argument to an OKBC operation. It will be expected to resolve to an
 individual, which may or may not be a frame.
 * :value - it is to be used as a value argument to
 an OKBC operation, such as put-slot-value.
 * The frame-action argument
 controls how the reading process interprets entities that can be interpreted
 as frames. The result-of-read value is false if an
 error occurs. The third value returned (completions-alist) is
 false if an error occurs, or otherwise is an association list of
 the form
 
 ((&lt;&lt;string1&gt;&gt; &lt;&lt;substring1&gt;&gt; &lt;&lt;frame1&gt;&gt; &lt;&lt;frame2&gt;&gt;... &lt;&lt;frameN&gt;&gt;)
 * (&lt;&lt;string2&gt;&gt; ....) ...)
 where &lt;&lt;stringX&gt;&gt; are strings found in string-or-value
 that match the frames &lt;&lt;frame1&gt;&gt; ... &lt;&lt;frameN&gt;&gt;
 (possibly by using any specified wildcards), and &lt;&lt;substringX&gt;&gt;
 are the corresponding
 longest matching initial substrings for each &lt;&lt;stringX&gt;&gt; (see the
 specification of get-frames-matching).
 
 * Wildcards-allowed-p
 * -- has the same meaning as in get-frames-matching. Wildcards are
 * interpreted piecewise for the components extracted from
 * string-or-value. Thus, "(fr* j*)" and
 * ("fr*" "j*") both denote a list
 * expression with two wildcarded components, and would match
 * (fred joe).
 * Force-Case-Insensitive-P
 * -- when true causes frame identification comparison to be
 * case-insensitive, irrespective of the preferred case sensitivity of the
 * KB itself.
 * Error-p
 * -- when true will signal a kb-value-read-error error if a
 * problem arises during the reading process, for example, due to
 * mismatched parentheses.
 * Frame-action
 * -- is a keyword from the following list:
 * 
 * :error-if-not-unique
 * -- If any substring is found that matches more than one frame then
 * signal a not-unique-error error.
 * :do-not-coerce-to-frames
 * -- Substrings of string-or-value (if a string), or strings
 * and symbols in string-or-value (if a nonstring) that
 * match frames are not converted into frames, but may be mapped
 * into strings or symbols.
 * :must-name-frames
 * -- Any symbol or string value must be coercible to a frame. If it
 * is not, a not-coercible-to-frame error is signaled.
 * :options-if-not-unique
 * -- For each ambiguous frame reference in string-or-value,
 * give the possible matches in an entry in the
 * completions-alist returned value.
 * For example, if in a KB there are frames called "FRED",
 "FREDDY", and "FRESH" and the call
 
 (coerce-to-kb-value "fr*" :frame-action :options-if-not-unique)
 is made, the values returned would be
 
 * 1.
 * false-- The coercion could not complete because of
 * the ambiguity.
 2.
 * true-- The operation completed without error.
 3.
 * (("FR*" "FRE" FRED FREDDY FRESH)) -- Only one ambiguous
 * reference was found, and for that the longest matching substring for
 * the pattern "FR*" is "FRE", and the matching frames
 * are {FRED, FREDDY, FRESH}.
 * See also get-frames-matching, which is called to identify frames.
 */

okbc_node *okbc_coerce_to_kb_value(const char *string_or_value,
				   const char *target_context,
				   const char *kb, int wildcards_allowed_p,
				   int force_case_insensitive_p,
				   int error_p, const char *frame_action,
				   int kb_local_only_p)
{
    okbc_node *ret = NULL;

    return ret;
}

/* okbc_coerce_to_slot(thing, kb, error_p, kb_local_only_p)
 *
 * (thing &key kb (error-p true) kb-local-only-p)
 *
 * Return Value(s): slot slot-found-p 
 * 
 * Flags: O R 
 * 
 * Coerces thing to a slot. This operation returns two values.
 * 
 * slot - If thing identifies a slot for kb,
 * then this value is the slot so identified, or
 * falseotherwise.
 * slot-found-p - If the slot is found then true,
 * otherwise false.
 * If error-p is trueand the slot is not found, then a
 * slot-not-found error is signaled.
 *
 * It is an error to call coerce-to-slot with error-p being
 * true, and with a value of thing that does not uniquely identify
 * a slot. If this happens, a not-unique-error error should be
 * signaled.
 *
 * Note that in some KRS, false may be a valid slot. No portable
 * program may assume that a returned value of false for the first
 * (slot) returned value implies that slot-found-p is false.
 */

okbc_node *okbc_coerce_to_slot(const char *thing, const char *kb,
			       int error_p, int kb_local_only_p)
{
    okbc_node *ret = NULL;

    return ret;
}
