/*
 *  Copyright (C) 2002 Steve Harris, ECS, University of Southampton
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  $Id: okbc_create.c,v 1.2 2003/02/20 19:01:15 theno23 Exp $
 */

#include <stdio.h>

#include "rdfsql/rdfsql.h"
#include "okbc_types.h"

/* okbc_create_class(name, kb, direct_types, direct_superclasses, primitive_p, doc, template_slots, template_facets, own_slots, own_facets, handle, pretty_name, kb_local_only_p)
 *
 * (name &key kb direct-types direct-superclasses (primitive-p true) doc template-slots template-facets own-slots own-facets handle pretty-name kb-local-only-p)
 *
 * Return Value(s): new-class 
 * 
 * Flags: O W 
 * 
 * Creates a class called name as a direct subclass
 * of the list of classes (or class) direct-superclasses. For KRSs
 * that support the distinction between primitive and nonprimitive concepts,
 * primitive-p specifies the primitiveness of the created class.
 * The parameters doc, template-slots,
 * template-facets, own-slots, own-facets, 
 * direct-types, handle, and pretty-name have the
 * same meaning as for create-frame. For KRSs that support metaclasses,
 * the direct-types argument specifies the type(s) of the class
 * to be created (i.e., metaclasses). Returns the new-class.
 */

okbc_node *okbc_create_class(const char *name, const char *kb,
			     const char *direct_types,
			     const char *direct_superclasses,
			     int primitive_p, const char *doc,
			     const char *template_slots,
			     const char *template_facets,
			     const char *own_slots, const char *own_facets,
			     const char *handle, const char *pretty_name,
			     int kb_local_only_p)
{
    okbc_node *ret = NULL;

    return ret;
}

/* okbc_create_facet(name, kb, frame_or_nil, slot_or_nil, slot_type, direct_types, doc, own_slots, own_facets, handle, pretty_name, kb_local_only_p)
 *
 * (name &key kb frame-or-nil slot-or-nil (slot-type :own) direct-types doc own-slots own-facets handle pretty-name kb-local-only-p)
 *
 * Return Value(s): new-facet 
 * 
 * Flags: M W 
 * 
 * Creates a facet called name on slot-or-nil that is
 * associated with frame-or-nil. If frame-or-nil is false,
 * the facet's frame domain is unconstrained (i.e., the facet may apply to
 * slot-or-nil in any frame).
 *
 * If slot-or-nil is false, the slot domain of the facet is
 * unconstrained (i.e., the facet may apply to all slots in
 * frame-or-nil, and if frame-or-nil is also false, may
 * apply to all slots in all frames.)
 * If :facet is a member of the behavior values for the
 * :are-frames behavior, direct-types,
 * doc, own-slots, own-facets, handle and
 * pretty-name have the same interpretation as for create-frame.
 * If either frame-or-nil or slot-or-nil is false,
 * slot-type is ignored. If either of the frame-or-nil or
 * slot-or-nil arguments is
 * false, and the KRS does not support facets with unconstrained domains,
 * a domain-required error will be signaled.
 * If facets must be uniquely named and a facet named name already
 * exists, a facet-already-exists error will be signalled.
 * Returns the new-facet.
 */

okbc_node *okbc_create_facet(const char *name, const char *kb,
			     const char *frame_or_nil,
			     const char *slot_or_nil,
			     const char *slot_type,
			     const char *direct_types, const char *doc,
			     const char *own_slots, const char *own_facets,
			     const char *handle, const char *pretty_name,
			     int kb_local_only_p)
{
    okbc_node *ret = NULL;

    return ret;
}

/* okbc_create_frame(name, frame_type, kb, direct_types, direct_superclasses, doc, template_slots, template_facets, own_slots, own_facets, primitive_p, handle, pretty_name, kb_local_only_p)
 *
 * (name frame-type &key kb direct-types direct-superclasses doc template-slots template-facets own-slots own-facets (primitive-p true) handle pretty-name kb-local-only-p)
 *
 * Return Value(s): new-frame 
 * 
 * Flags: O W 
 * 
 * Creates a new frame called name of type frame-type.
 * Frame-type is one of {:class, :slot, :facet,
 * :individual}. A call to create-frame is equivalent to
 * a call to one of create-class, create-individual,
 * create-slot, or create-facet passing through the appropriate
 * arguments, depending on the value of frame-type. If
 * frame-type is either :slot or :facet, the slot
 * (or facet) created will have unconstrained domains.
 *
 * If the :frame-names-required behavior has the value false for
 * kb, new-name may be false. If the
 * :frame-names-required behavior is true for kb,
 * new-name must uniquely name the new frame, and a
 * frame-already-exists error will be signaled if new-name
 * is coercible to an existing frame.
 *
 * Direct-types is a list of classes (or class) of which this new
 * frame is to be a direct instance.
 * Direct-superclasses is a list of classes (or class) of which the
 * new frame is to be a direct subclass.
 * Doc, if specified, is a string documenting the new frame.
 * Pretty-name is the pretty-name of the new frame. Returns
 * new-frame, which identifies the newly created frame.
 *
 * Template-slots and own-slots each take a list of slot 
 * specifications. A slot specification assigns a set of values to a
 * slot. The syntax of a slot specification is
 * 
 * slot-spec ::= (slot slot-value-spec*)
 * slot-value-spec ::= default-slot-value | slot-value
 * default-slot-value ::= (:default slot-value)
 * where slot identifies a slot, or names a slot to be created. If
 * slot already exists, it is simply attached to the new frame, if
 * it does not currently exist, it is created and attached to the new frame.
 * Each slot-value is an entity suitable as a value of
 * the specified slot. Default slot values are identified by appearing in
 * a list whose first element is :default. Template slots are only
 * allowed for class frames - that is, when frame-type is
 * :class.
 * 
 * Template-facets and own-facets each take a list of facet 
 * specifications, which can assign a set of facet values. A facet
 * specification has the form:
 * 
 * facet-spec ::= (slot fspec*)
 * fspec ::= (facet facet-value-spec*)
 * facet-value-spec ::= default-facet-value | facet-value
 * default-facet-value ::= (:default facet-value)
 * where slot identifies a slot, or names a slot to be created. If
 * slot already exists, it is simply attached to the new frame, if
 * it does not currently exist, it is created and attached to the new frame.
 * Facet identifies a facet, or names a facet to be created. If
 * facet already exists, it is simply attached to slot on
 * the new frame, if it does not currently exist, it is created and attached
 * to slot on the new frame. Each facet-value is an object
 * suitable as a value of the specified facet. Default facet values are
 * identified by appearing in a list whose first element is :default. 
 * Template facets are allowed only for class frames - that is, when
 * frame-type is :class.
 *
 * All slot and facet names in slot and facet specs are defined in a unified
 * namespace that operates across all of the :own-slots,
 * :own-facets, :template-slots, and :template-facets
 * arguments. Thus, in the following example, all occurrences of the slot
 * s1 and the facet f1 denote the same slot and facet
 * respectively.
 *
 * The values specified in slot and facet specifications are interpreted
 * conjunctively. Thus, in the following example, the slot
 * s1 will have three values; 42, 100 and 2001, rather than just
 * the value 2001.
 * 
 * (create-frame 'foo :class
 * :own-slots '((s1 42 100)
 * (s1 2001))
 * :own-facets '((s1 (:value-type :integer))
 * (s1 (f1 "Own hello")))
 * :template-facets '((s1 (f1 "Template hello"))))
 *
 * Primitive-p may be used only when creating a class.
 * When primitive-p is false, the KRS will make
 * the class nonprimitive, if possible.
 *
 * Handle, if supplied, is a previously allocated frame handle for the
 * new frame to be created. This is used by network applications, and
 * operations such as copy-frame and copy-kb. (See
 * allocate-frame-handle.) It is an error to supply a value for the
 * handle argument that is already coercible to a frame. If this
 * occurs, a frame-already-exists error should be signaled.
 *
 * Note that if frame-type is either :slot or :facet,
 * then a frame might not be created because slots (or facets) might not
 * be represented as frames in kb. If this is the case, and slots
 * (or facets) with unconstrained domains are not supported, a
 * domain-required error will be signaled.
 *
 * It is an error to supply own-slots, own-facets if a frame
 * will not be created, according to the :are-frames behavior, and a
 * not-a-frame-type error should be signaled.
 */

okbc_node *okbc_create_frame(const char *name, const char *frame_type,
			     const char *kb, const char *direct_types,
			     const char *direct_superclasses,
			     const char *doc, const char *template_slots,
			     const char *template_facets,
			     const char *own_slots, const char *own_facets,
			     int primitive_p, const char *handle,
			     const char *pretty_name, int kb_local_only_p)
{
    okbc_node *ret = NULL;

    return ret;
}

/* okbc_create_individual(name, kb, direct_types, doc, own_slots, own_facets, handle, pretty_name, kb_local_only_p)
 *
 * (name &key kb direct-types doc own-slots own-facets handle pretty-name kb-local-only-p)
 *
 * Return Value(s): new-individual 
 * 
 * Flags: O W 
 * 
 * Creates an individual called name. The one or more classes 
 * that are the direct types of the instance are given by
 * direct-types. 
 * The parameters doc, own-slots, own-facets,
 * handle, and pretty-name all have the same meaning as for
 * create-frame. Returns new-individual, which identifies the
 * new frame.
 */

okbc_node *okbc_create_individual(const char *name, const char *kb,
				  const char *direct_types,
				  const char *doc, const char *own_slots,
				  const char *own_facets,
				  const char *handle,
				  const char *pretty_name,
				  int kb_local_only_p)
{
    okbc_node *ret = NULL;

    return ret;
}

/* okbc_create_kb(name, kb_type, kb_locator, initargs, connection)
 *
 * (name &key kb-type kb-locator initargs (connection (local-connection)))
 *
 * Return Value(s): new-kb 
 * 
 * Flags: O W 
 * 
 * Creates a new KB (see Section 2.9) called name whose
 * implementation type is kb-type. Kb-type identifies the
 * underlying KRS that will be used to manipulate the KB. Returns the
 * new-kb.
 *
 * Note that this operation creates a new in-memory KB; it does
 * not necessarily create a persistent version of the knowledge base on
 * external storage until save-kb or save-kb-as is called.
 *
 * kb-locator, if supplied, describes the new KB. Kb-locators can be
 * created using create-kb-locator.
 * If kb-locator is not supplied, a default kb-locator will be assigned
 * by the KRS for kb-type and connection.
 *
 * Initargs is a list of initializations for the new KB as
 * required by the kb-type. The mechanism underlying the
 * implementation of create-kb is not specified and the user cannot,
 * therefore, rely on any underlying native object system initialization
 * protocol being invoked. The format and content of the initialization
 * arguments will be documented with the kb-type. For example,
 * if the KB being created allows the specification of parent (included) KBs,
 * a set of initialization arguments might be as follows:
 * 
 * (list :parent-kbs (list my-kb))
 * Any KB created with create-kb can be found by using
 * either find-kb or find-kb-of-type, and it is included in
 * the values returned by get-kbs. A KB created with
 * create-kb is a frame object in the meta-kb.
 *
 * Implementation note: It is the responsibility of the implementations of
 * create-kb to register new KBs in the Meta KB (for example, by using
 * put-instance-types to tell the Meta KB that the new KB is an instance
 * of kb-type.
 */

okbc_node *okbc_create_kb(const char *name, const char *kb_type,
			  const char *kb_locator, const char *initargs,
			  const char *connection)
{
    okbc_node *ret = NULL;

    return ret;
}

/* okbc_create_kb_locator(thing, kb_type, connection)
 *
 * (thing &key kb-type (connection (local-connection)))
 *
 * Return Value(s): kb-locator 
 * 
 * Flags: M R 
 * 
 * Returns a new kb-locator associated with thing for a kb of
 * type kb-type. If thing is a KB, the kb-locator created is
 * associated with that KB in the meta-kb. It is an error for
 * thing to be an incomplete description of a kb-locator.
 *
 * Thing is a kb-type and connection specific
 * specification of a KB location sufficient to create and fully
 * initialize a KB locator.
 *
 * For example, thing may identify the pathname for a KB that
 * resides in a disk file. Each back-end implementation must provide
 * documentation for all values of thing that the kb-type
 * and connection will accept other than KBs, which are always
 * accepted.
 *
 * Implementation note: Back end implementators may use any legal
 * OKBC value entity for the thing argument as long as it
 * consists only of the primitive data types: integer, float, string,
 * symbol, true, false, or list. Values of thing of these data
 * types will always be transmitted by networked implementations without
 * substitution of remote references. For example, the following
 * could be a legal value for for the thing argument for some
 * kb-type
 * 
 * (:db-file "/projects/foo/my-database.data" :db-type :oracle :name my-kb)
 */

okbc_node *okbc_create_kb_locator(const char *thing, const char *kb_type,
				  const char *connection)
{
    okbc_node *ret = NULL;

    return ret;
}

/* okbc_create_procedure(kb, arguments, body, environment)
 *
 * (&key kb arguments body environment)
 *
 * Return Value(s): procedure 
 * 
 * Flags: O R 
 * 
 * Defines and returns a procedure in the OKBC procedure language. The
 * arguments are defined as follows:
 * 
 * arguments - the argument list for the procedure. The
 * argument list can be expressed in one of three forms.
 * 
 * 1.
 * A list of symbols
 * 2.
 * A string that is the printed representation of a list
 * of symbols
 * 3.
 * false- the null argument list
 * For example, the argument lists (a b c), and
 * "(a b c)" are equivalent, as are "()" and false.
 * The string representation is provided for language bindings in
 * which it may be inconvenient to create lists of symbols.
 * body - The body for the procedure expressed in the syntax
 * defined in section 5. The body can be
 * provided in one of two forms:
 * 
 * 1.
 * A body-form
 * 2.
 * A list of body-forms
 * 3.
 * A string that is the printed representation of a sequence
 * of body-forms
 * For example, the following procedure bodies are equivalent:
 * 
 * ((put-slot-values frame slot values :slot-type :own)
 * (get-slot-value frame slot :slot-type :own))
 * and
 * 
 * "(put-slot-values frame slot values :slot-type :own)
 * (get-slot-value frame slot :slot-type :own)"
 * The string representation is provided for language bindings in
 * which it may be inconvenient to create the complex list structure
 * required in the procedure language.
 * environment - A predefined set of bindings between variables
 * mentioned in the procedure body and their associated values.
 * The environment is a list of 2-tuples of the form
 * 
 * ((var1 value1)
 * (var2 value2)
 * ....
 * (varn valuen))
 * where varN are the variables mentioned in body, and
 * valueN are the associated values for the variables.
 * A procedure is a legal argument to any OKBC operator in a position that
 * expects a procedure.
 * For example,
 * 
 * (call-procedure
 * #'(lambda (frame) (get-frame-pretty-name frame :kb kb))
 * :kb kb :arguments (list my-frame))
 * and
 * 
 * (call-procedure
 * (create-procedure :arguments '(frame)
 * :body '(get-frame-pretty-name frame :kb kb))
 * :kb my-kb :arguments (list my-frame))
 * are semantically identical.
 *
 * The main differences between
 * procedures and lambda expressions in Lisp are as follows:
 * 
 * 1.
 * All bindings in procedures are dynamic, not lexical.
 * 2.
 * Only a restricted set of operations is available in procedures.
 * 3.
 * Lambda defines a lexical closure over any free references.
 * procedure defines a dynamic closure over its free
 * references. The environment of the procedure is
 * prefilled with bindings for the names of the arguments to
 * the OKBC operator in which it is being executed. In the above
 * case, call-procedure takes arguments KB,
 * Arguments, and Kb-local-only-p
 * which will take on the values my-kb, (my-frame), and
 * nil (the default), respectively.
 * 4.
 * Lambda expressions are meaningful only within the Lisp system
 * in which the OKBC system is running. procedures are
 * executable on any (possibly network-connected) OKBC KB.
 * 5.
 * procedures are package-insensitive in all respects
 * other than quoted constants.
 * 
 * Note that persistent side effects to &lt;&lt;var1&gt;&gt; and &lt;&lt;var2&gt;&gt;
 * cannot be made from within the procedure. The arguments and variables
 * mentioned in the procedure exist in a different space from the variables
 * in a user program. The only ways to establish associations between values
 * in a user program and variables in a procedure are through the use of the
 * environment argument to create-procedure, or by the
 * arguments argument to call-procedure.
 */

okbc_node *okbc_create_procedure(const char *kb, const char *arguments,
				 const char *body, const char *environment)
{
    okbc_node *ret = NULL;

    return ret;
}

/* okbc_create_slot(name, kb, frame_or_nil, slot_type, direct_types, doc, own_slots, own_facets, handle, pretty_name, kb_local_only_p)
 *
 * (name &key kb frame-or-nil (slot-type :all) direct-types doc own-slots own-facets handle pretty-name kb-local-only-p)
 *
 * Return Value(s): new-slot 
 * 
 * Flags: M W 
 * 
 * Creates a slot called name in the frame specified by
 * frame-or-nil. Returns the new-slot. If the slot to be
 * created is to be represented as a frame (:slot is a member of the
 * :are-frames behavior), direct-types,
 * doc, own-slots, own-facets, handle, and
 * pretty-name have the same interpretation as for create-frame.
 * If frame-or-nil is false, slot-type is ignored, and the
 * slot's domain is ignored. If the
 * frame argument is false, and the KRS does not support slots with
 * unconstrained domains, a domain-required error will be signaled.
 * If slots must be uniquely named and a slot named name already
 * exists, a slot-already-exists error will be signalled.
 */

okbc_node *okbc_create_slot(const char *name, const char *kb,
			    const char *frame_or_nil,
			    const char *slot_type,
			    const char *direct_types, const char *doc,
			    const char *own_slots, const char *own_facets,
			    const char *handle, const char *pretty_name,
			    int kb_local_only_p)
{
    okbc_node *ret = NULL;

    return ret;
}
