/*
 *  Copyright (C) 2002 Steve Harris, ECS, University of Southampton
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  $Id: okbc_delete.c,v 1.2 2003/02/20 19:01:16 theno23 Exp $
 */

#include <stdio.h>

#include "rdfsql/rdfsql.h"
#include "okbc_types.h"

/* okbc_delete_facet(facet, kb, kb_local_only_p)
 *
 * (facet &key kb kb-local-only-p)
 *
 * Return Value(s): void 
 * 
 * Flags: M W 
 * 
 * Deletes the facet from all frames containing that facet, and the facet frame
 * itself if the facet is represented as a frame. As a result of
 * delete-facet, facet will return false for calls to
 * facet-p, and facet is not returned by any of the
 * facet-returning operations, such as get-kb-facets and
 * get-slot-facets. It is no longer possible to access any values of
 * facet. Returns no values.
 *
 * Many implementations may, in fact, delete the values associated the
 * facet in frames as well as making the facet no longer facet-p. Other
 * implementations will simply make these values inaccessible.
 */

void okbc_delete_facet(const char *facet, const char *kb,
		       int kb_local_only_p)
{
}

/* okbc_delete_frame(frame, kb, kb_local_only_p)
 *
 * (frame &key kb kb-local-only-p)
 *
 * Return Value(s): void 
 * 
 * Flags: M W 
 * 
 * Deleting a frame from a KB is difficult to specify in a portable way.
 * After calling delete-frame, the frame argument
 * will no longer be a valid frame reference (frame-p will return
 * false). As a consequence, the value of frame will
 * not be a valid argument to any OKBC operation requiring a frame reference,
 * such as get-frame-slots. It will no longer be possible to access
 * any of the properties (e.g., slots, facets) of
 * frame. Implementations will delete at least the
 * properties documented as being returned by get-frame-details
 * from the kb.
 *
 * Note that after a call to delete-frame, references
 * to frame may still remain in the KB. Returns no values.
 */

void okbc_delete_frame(const char *frame, const char *kb,
		       int kb_local_only_p)
{
}

/* okbc_delete_slot(slot, kb, kb_local_only_p)
 *
 * (slot &key kb kb-local-only-p)
 *
 * Return Value(s): void 
 * 
 * Flags: M W 
 * 
 * Deletes the slot from all frames containing that slot, and the slot frame
 * itself if the slot is represented as a frame. As a result of
 * delete-slot, slot will return false for calls to
 * slot-p, and slot is not returned by any of the
 * slot-returning operations, such as get-kb-slots and
 * get-frame-slots. It is no longer possible to access any values of
 * slot or any facets or facet values on slot.
 * Returns no values.
 *
 * Many implementations may, in fact, delete the values associated the
 * slot in frames as well as making the slot no longer slot-p. Other
 * implementations will simply make these values inaccessible.
 */

void okbc_delete_slot(const char *slot, const char *kb,
		      int kb_local_only_p)
{
}
