/*
 *  Copyright (C) 2002 Steve Harris, ECS, University of Southampton
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  $Id: okbc_misc.c,v 1.3 2003/02/20 19:01:17 theno23 Exp $
 */

#include <stdio.h>

#include "rdfsql/rdfsql.h"
#include "okbc.h"

/* okbc_all_connections()
 *
 * ()
 *
 * Return Value(s): list-of-connections 
 * 
 * Flags: E R 
 * 
 * Returns a list of all of the known connections.
 */

okbc_node *okbc_all_connections()
{
    okbc_node *ret = NULL;

    return ret;
}

/* okbc_allocate_frame_handle(frame_name, frame_type, kb, frame_handle)
 *
 * (frame-name frame-type &key kb frame-handle)
 *
 * Return Value(s): frame-handle 
 * 
 * Flags: O W 
 * 
 * Allocates a frame handle in kb. It is not anticipated that this
 * operation will be called by OKBC applications, but rather by OKBC back end
 * implementations. This operation can be used in two distinct ways:
 * 
 * 1.
 * Given a frame located in an arbitrary KB, typically different
 * from kb, passing its frame-name, frame-type, and
 * frame-handle will return a frame handle to represent that frame if
 * such a frame were to be created in kb. This is useful in OKBC
 * operations such as copy-frame and copy-kb where it is often
 * necessary to make forward references to frame objects. 
 * 2.
 * Providing just a frame-name and a frame-type
 * will return a frame handle to represent that frame if such a frame were
 * to be created in kb. This is useful when an implementation wants
 * to allocate a frame handle either during the frame creation process, or
 * to create forward references to frames when faulting them in from a
 * lazy persistent store. Frame-name may be false.
 * Frame-type is the type of the frame as identified by the operation
 * get-frame-type; that is, it must be in the set {:class,
 * :individual, :slot, :facet}.
 *
 * The rationale for the arguments to this operation is as follows:
 * 
 * frame-name - In some KRSs, the name of a frame cannot be
 * changed after the frame handle has been allocated. OKBC therefore
 * mandates that the name be supplied. If the
 * :frame-names-required behavior has the value false,
 * this argument may be false.
 * frame-type - In some KRSs, the type of data structure used
 * to represent the frame handles of (say) classes is completely
 * different from that of (say) individual frames. OKBC therefore
 * mandates that the frame type be specified. Implementations that use
 * the same representation for all frame handles will be able to ignore
 * this argument, but it is not portable.
 * frame-handle - Some KRSs may choose to use a frame handle
 * provided as the value of the frame-handle argument as the
 * new frame handle. This allows implementations that do not have a
 * special data structure for frame handles to save memory and to
 * maximize the correspondence between the objects in different KBs.
 * The contract of allocate-frame-handle does not require the callee to
 * return the same frame handle if called multiple times with identical
 * arguments. Note that this is particularly important in case 2, above,
 * with frame-name being false. It is the responsibility of
 * the caller to remember the
 * correspondence between its frames and the frame handles allocated. A frame
 * handle allocated using allocate-frame-handle can be used as the
 * value of the handle argument to create-frame,
 * create-class, create-slot, create-facet, and
 * create-individual. During the execution of these operations, it is
 * the responsibility of the kb to preserve any necessary object
 * identity so that, for example,
 * 
 * new-handle = allocate-frame-handle(name, :class, kb, handle);
 * new-frame = create-class(name, .... :handle new-handle);
 * new-handle == get-frame-handle(new-frame) // this identity must hold!
 */

okbc_node *okbc_allocate_frame_handle(const char *frame_name,
				      const char *frame_type,
				      const char *kb,
				      const char *frame_handle)
{
    okbc_node *ret = NULL;

    return ret;
}

/* okbc_askable(sentence, kb, value_selector, kb_local_only_p)
 *
 * (sentence &key kb (value-selector :either) kb-local-only-p)
 *
 * Return Value(s): boolean 
 * 
 * Flags: O R 
 * 
 * The askable operation returns false if the KRS can
 * determine that asking the sentence would result in a
 * cannot-handle error being signaled, and true otherwise.
 * It may also signal the syntax-error condition. Even if
 * askable returns true,
 * ask may still not be able to handle the sentence.
 */

okbc_node *okbc_askable(const char *sentence, const char *kb,
			const char *value_selector, int kb_local_only_p)
{
    okbc_node *ret = NULL;

    return ret;
}

/* okbc_ask(query, kb, reply_pattern, inference_level, number_of_values, error_p, where, timeout, value_selector, kb_local_only_p)
 *
 * (query &key kb (reply-pattern query) (inference-level :taxonomic) (number-of-values :all) (error-p true) where timeout (value-selector :either) kb-local-only-p)
 *
 * Return Value(s): reply-pattern-list exact-p more-status 
 * 
 * Flags: E O R 
 * 
 * Asks a query of the OKBC kb. Query may be any
 * sentence in the OKBC Assertion Language that is askable. A
 * cannot-handle error
 * may be signaled if it is not askable. Reply-pattern is an
 * expression mentioning KIF variables contained in query.
 *
 * Reply-pattern is any list structure mentioning the variables
 * in the query, or just the name of a variable. For example, consider
 * a query that is a sentence of the form,
 * 
 * (subclass-of ?x ?y)
 * that is, find me the things that are subclasses of other things. If there
 * is a match in the KB for ?x = human and ?y = animal.
 * - the class human is a subclass of the class animal - then
 * if the reply-pattern were to be
 * 
 * (superclass-of ?y ?x)
 * we would be returned a list of sentences of which one
 * would be (superclass-of animal human). The explicit use of a reply
 * pattern in this manner allows the user to get either sentences
 * that can be conveniently reasserted using tell, or tuples of
 * matches in a shape that is convenient to the application.
 *
 * When error-p is true, any errors resulting from the execution of
 * the query are signaled. When error-p is false, all possible attempts are
 * made to continue with the query and deliver as many results as were
 * requested.
 *
 * If the resources used by ask are a concern, the time (in
 * seconds) allowed to answer a query will be limited, if possible, as
 * specified by timeout. If the value of timeout is
 * false, an unlimited time is allowed for ask to complete.
 *
 * The where clause can be used to specify a list of bindings to
 * be used for any variables appearing in the query. During query
 * evaluation, such variables are replaced by the values specified by
 * where. A valid value of where is a list of 2-tuples,
 * with each tuple consisting of a variable and value pair.
 *
 * Ask returns three values.
 * 
 * 1.
 * reply-pattern-list - In this list, each element is an
 * instance of reply-pattern, with all variables mentioned in
 * query substituted.
 * 2.
 * exact-p - This has its normal interpretation.
 * 3.
 * more-status - This has its normal interpretation, except
 * that an additional option :timeout may be returned for the
 * more-status value by ask if the call terminates because
 * execution time exceeds the limit specified by the timeout
 * argument.
 * When ask is given a syntactically invalid query, it
 * signals the syntax-error error. When ask realizes that
 * the query cannot be handled by the KRS, it signals a
 * cannot-handle error.
 *
 * The following query matches four channel oscilloscopes with a
 * bandwidth greater than 80MHz. It returns a list of pairs (?osc ?bandwidth) satisfying the query.
 * 
 * (ask '(and (oscilloscope ?osc)
 * (number-of-channels ?osc ?chans)
 * (= ?chans 4)
 * (bandwidth ?osc ?bandwidth)
 * (&gt; ?bandwidth (* 80 mega-hertz)))
 * :reply-pattern '(?osc ?bandwidth)
 * :number-of-values 10 :kb kb)
 * All KIF operators in the query are parsed in a
 * package-insensitive manner. For example, (and A B) and (:and A B) have the same effect. Object, relation, and function constant
 * references in query are taken as arguments to 
 * get-frames-matching. Frame references in the query must uniquely 
 * identify frames. (See get-frames-matching.)
 */

okbc_node *okbc_ask(const char *query, const char *kb,
		    const char *reply_pattern, const char *inference_level,
		    int number_of_values, int error_p,
		    const char *where, const char *timeout,
		    const char *value_selector, int kb_local_only_p)
{
    okbc_node *ret = NULL;

    return ret;
}

/* okbc_call_procedure(procedure, kb, arguments)
 *
 * (procedure &key kb arguments)
 *
 * Return Value(s): value 
 * 
 * Flags: E O R 
 * 
 * Returns the value resulting from applying
 * procedure to
 * arguments. See section 5 for a definition of
 * procedures.
 */

okbc_node *okbc_call_procedure(const char *procedure, const char *kb,
			       const char *arguments)
{
    okbc_node *ret = NULL;

    return ret;
}

/* okbc_class_p(thing, kb, kb_local_only_p)
 *
 * (thing &key kb kb-local-only-p)
 *
 * Return Value(s): boolean 
 * 
 * Flags: M R 
 * 
 * Returns true if thing identifies a class.
 */

okbc_node *okbc_class_p(const char *thing, const char *kb,
			int kb_local_only_p)
{
    okbc_node *ret = NULL;

    return ret;
}

/* okbc_coercible_to_frame_p(thing, kb, kb_local_only_p)
 *
 * (thing &key kb kb-local-only-p)
 *
 * Return Value(s): boolean 
 * 
 * Flags: O R 
 * 
 * Returns true when thing can be coerced to a frame
 * by using coerce-to-frame, and otherwise returns false.
 */

okbc_node *okbc_coercible_to_frame_p(const char *thing, const char *kb,
				     int kb_local_only_p)
{
    okbc_node *ret = NULL;

    return ret;
}

/* okbc_connection_p(thing)
 *
 * (thing)
 *
 * Return Value(s): boolean 
 * 
 * Flags: R 
 * 
 * Is true if thing is a connection, and false otherwise.
 */

okbc_node *okbc_connection_p(const char *thing)
{
    okbc_node *ret = NULL;

    return ret;
}

/* okbc_continuable_error_p(thing)
 *
 * (thing)
 *
 * Return Value(s): boolean 
 * 
 * Flags: R 
 * 
 * Returns true if thing is a continuable error, and false otherwise. An error is said to be continuable only if the state of the KB
 * is known not to have been damaged by the error in such a way that the
 * behavior of subsequent OKBC operations becomes undefined. Thus, although
 * the signalling of a continuable error will interrupt any processing
 * currently being performed, subsequent OKBC calls will be well defined.
 * After a noncontinuable error, the state of the KB and the behavior of the
 * KRS and application are undefined.
 */

okbc_node *okbc_continuable_error_p(const char *thing)
{
    okbc_node *ret = NULL;

    return ret;
}

/* okbc_current_kb()
 *
 * ()
 *
 * Return Value(s): kb 
 * 
 * Flags: R 
 * 
 * Returns the current KB. The current KB is set using goto-kb.
 */

okbc_node *okbc_current_kb()
{
    okbc_node *ret = NULL;

    return ret;
}

/* okbc_decontextualize(value, from_context, kb)
 *
 * (value from-context &key kb)
 *
 * Return Value(s): decontextualized-value 
 * 
 * Flags: M R 
 * 
 * Given a value from kb, returns a
 * decontextualized-value, which contains no KB or KRS-specific
 * data structures. In particular,
 * any references to frame objects will be replaced with KRS-independent frame
 * handles (produced using frs-independent-frame-handle), and all
 * values outside the standard set of OKBC data types that have
 * no interpretation outside kb will be
 * replaced with remote-value references. Any frame references that are
 * the result of an KRS-specific mapping of a canonically named frame will be
 * replaced with the canonical name. Thus, for example, a facet
 * frame called cardinality-of-slot would be mapped back to a frame
 * handle for the canonical facet-reference :cardinality.
 *
 * From-context is one of {:frame, :slot,
 * :facet, :value}. It identifies the context of the argument
 * to be decontextualized. For example, if the
 * decontextualization is to be applied to a slot value, then
 * from-context should be :value. If the
 * decontextualization is to be applied to a slot (i.e., something that would
 * be used as a slot argument to an operation such as
 * get-slot-values), then from-context should be :slot.
 *
 * It is not anticipated that this operation will be called by OKBC
 * applications, but rather by OKBC back end implementations. It is used to
 * ensure correct operation in networked applications and during communication
 * between KBs of different kb-types.
 */

okbc_node *okbc_decontextualize(const char *value,
				const char *from_context, const char *kb)
{
    okbc_node *ret = NULL;

    return ret;
}

/* okbc_eql_in_kb(arg0, arg1, kb, kb_local_only_p)
 *
 * (arg0 arg1 &key kb kb-local-only-p)
 *
 * Return Value(s): boolean 
 * 
 * Flags: O R 
 * 
 * Returns true iff arg0 and arg1
 * identify the same frame in kb, or are the same object (==, EQLness),
 * and otherwise returns false.
 * Arg0 and arg1 may be any combination of objects coercible
 * to frames.
 */

okbc_node *okbc_eql_in_kb(const char *arg0, const char *arg1,
			  const char *kb, int kb_local_only_p)
{
    okbc_node *ret = NULL;

    return ret;
}

/* okbc_equal_in_kb(arg0, arg1, kb, kb_local_only_p)
 *
 * (arg0 arg1 &key kb kb-local-only-p)
 *
 * Return Value(s): boolean 
 * 
 * Flags: O R 
 * 
 * Returns true iff arg0 and arg1
 * identify the same frame in kb, or are the same object (==, EQLness),
 * or they are strings containing the same characters (case sensitively), or
 * both are lists with the same structure, and each of the elements
 * recursively is true according to equal-in-kb. Returns false otherwise.
 */

okbc_node *okbc_equal_in_kb(const char *arg0, const char *arg1,
			    const char *kb, int kb_local_only_p)
{
    okbc_node *ret = NULL;

    return ret;
}

/* okbc_equalp_in_kb(arg0, arg1, kb, kb_local_only_p)
 *
 * (arg0 arg1 &key kb kb-local-only-p)
 *
 * Return Value(s): boolean 
 * 
 * Flags: O R 
 * 
 * Returns true iff arg0 and arg1
 * identify the same frame in kb, or are the same object (==, EQLness),
 * or they are strings containing the same characters (case-insensitively), or
 * both are lists with the same structure, and each of the elements
 * recursively is true according to equalp-in-kb. Returns
 * false otherwise.
 */

okbc_node *okbc_equalp_in_kb(const char *arg0, const char *arg1,
			     const char *kb, int kb_local_only_p)
{
    okbc_node *ret = NULL;

    return ret;
}

/* okbc_establish_connection(connection_type, initargs)
 *
 * (connection-type &key initargs)
 *
 * Return Value(s): connection 
 * 
 * Flags: O R 
 * 
 * Establishes and returns a connection of type connection-type.
 * Initargs are initialization arguments for the connection if one
 * is created, are used to initialize the connection in a manner
 * specific to the connection type, and are documented with the definition
 * of the connection type itself. No guarantee is made that the connection
 * will be newly created. An existing, open connection with the same
 * initializations may be returned.
 *
 * For example, to initialize some form of network connection, the value of
 * initargs might be a property list of the form
 * (:host "my-host" :port 1234 :username "joe").
 *
 * Although the format of initargs is implementation-specific, OKBC
 * nevertheless mandates a set of standard names for commonly used
 * initializations.
 * 
 * HOST
 * - A string naming the host on which the server is
 * to be found
 * PORT
 * - An integer indicating a TCP/IP port on which the
 * server is to be found
 * USERNAME
 * - A string for the login name of the user on
 * the OKBC server
 * PASSWORD
 * - The password of the user on the server
 * 
 * Establishing a local connection requires no initialization arguments and
 * can be done more conveniently using local-connection.
 */

okbc_node *okbc_establish_connection(const char *connection_type,
				     const char *initargs)
{
    okbc_node *ret = NULL;

    return ret;
}

/* okbc_expunge_kb(kb_locator, kb_type, connection, error_p)
 *
 * (kb-locator &key kb-type (connection (local-connection)) (error-p true))
 *
 * Return Value(s): void 
 * 
 * Flags: M W 
 * 
 * Given a kb-locator, removes the KB identified by that locator
 * and any backup copies of it from permanent storage. Returns no values.
 * Any currently open KB identified by the locator will be unaffected,
 * and may be saved to other locations using save-kb-as. If
 * error-p is false, expunge-kb catches errors that
 * occur, and attempts to continue with the deletion process.
 */

void okbc_expunge_kb(const char *kb_locator, const char *kb_type,
		     const char *connection, int error_p)
{
}

/* okbc_fetch(enumerator, number_of_values)
 *
 * (enumerator &key (number-of-values :all))
 *
 * Return Value(s): list-of-values 
 * 
 * Flags: O R 
 * 
 * Returns a list-of-values of at most
 * number-of-values values remaining in the enumerator. If the
 * enumerator was exhausted before the call,
 * an enumerator-exhausted error will be signaled. Note that unlike
 * other operations taking a number-of-values argument, this operation
 * does not return a more-status value.
 */

okbc_node *okbc_fetch(const char *enumerator, int number_of_values)
{
    okbc_node *ret = NULL;

    return ret;
}

/* okbc_follow_slot_chain(frame, slot_chain, kb, union_multiple_values, inference_level, value_selector, kb_local_only_p)
 *
 * (frame slot-chain &key kb (union-multiple-values true) (inference-level :taxonomic) (value-selector :either) kb-local-only-p)
 *
 * Return Value(s): values 
 * 
 * Flags: O R 
 * 
 * Allows a program to traverse a chain of slot references,
 * gathering own slot values. For example, imagine that we wish to determine
 * the sisters of the father of the mother of John. The following two calls
 * accomplish this goal:
 * 
 * (follow-slot-chain 'john '(mother father sisters))
	
 * (get-slot-values
 * (get-slot-value
 * (get-slot-value 'john 'mother)
 * 'father)
 * 'sisters)
 * This operation is complicated by the fact that slots can have multiple
 * values. For example, imagine that John has two mothers--adopted and
 * biological. If union-multiple-values is false and a slot
 * has more than one value, a cardinality-violation error is signaled;
 * if true, then the slot chain becomes a tree, and the union of
 * all values found at the leaves of the tree is returned.
 */

okbc_node *okbc_follow_slot_chain(const char *frame,
				  const char *slot_chain, const char *kb,
				  const char *union_multiple_values,
				  const char *inference_level,
				  const char *value_selector,
				  int kb_local_only_p)
{
    okbc_node *ret = NULL;

    return ret;
}

/* okbc_free(enumerator)
 *
 * (enumerator)
 *
 * Return Value(s): void 
 * 
 * Flags: O R 
 * 
 * Indicates that the enumerator will no longer be used. The
 * enumerator and any cache of unseen values may be thrown away.
 * After calling free, it is an error to provide enumerator
 * as an argument to any operation, and if this is done, an
 * object-freed error should be signaled. It is especially important
 * to call free in a network setting when a program has finished
 * with the enumerator and its values have not been exhausted, so that
 * the server can reclaim space allocated to the enumerator.
 * Returns no values.
 */

void okbc_free(const char *enumerator)
{
}

/* okbc_goto_kb(kb)
 *
 * (kb)
 *
 * Return Value(s): void 
 * 
 * Flags: O W 
 * 
 * Makes kb the current KB. After a call to goto-kb, the value
 * of a call to current-kb will be kb. The newly established
 * current-kb will be used as the default value for the kb
 * argument by language bindings that support argument defaulting. Returns
 * no values.
 */

void okbc_goto_kb(const char *kb)
{
}

/* okbc_has_more(enumerator)
 *
 * (enumerator)
 *
 * Return Value(s): boolean 
 * 
 * Flags: O R 
 * 
 * Returns true if the enumerator has more values, otherwise
 * returns false.
 */

okbc_node *okbc_has_more(const char *enumerator)
{
    okbc_node *ret = NULL;

    return ret;
}

/* okbc_individual_p(thing, kb, kb_local_only_p)
 *
 * (thing &key kb kb-local-only-p)
 *
 * Return Value(s): boolean 
 * 
 * Flags: O R 
 * 
 * Returns true if thing identifies an individual entity, and
 * returns false if thing identifies a class.
 */

okbc_node *okbc_individual_p(const char *thing, const char *kb,
			     int kb_local_only_p)
{
    okbc_node *ret = NULL;

    return ret;
}

/* okbc_instance_of_p(thing, class, kb, inference_level, kb_local_only_p)
 *
 * (thing class &key kb (inference-level :taxonomic) kb-local-only-p)
 *
 * Return Value(s): boolean exact-p 
 * 
 * Flags: O R 
 * 
 * Returns true if thing is an instance of class, otherwise
 * returns false.
 */

okbc_node *okbc_instance_of_p(const char *thing, const char *class,
			      const char *kb, const char *inference_level,
			      int kb_local_only_p)
{
    okbc_node *ret = NULL;

    return ret;
}

/* okbc_local_connection()
 *
 * ()
 *
 * Return Value(s): connection 
 * 
 * Flags: R 
 * 
 * Returns a connection to the local OKBC implementation.
 */

okbc_node *okbc_local_connection()
{
    okbc_node *ret = NULL;

    okbc_intl_db = rs_connect("rdf", "rdf", "rdf", NULL);

    return ret;
}

/* okbc_meta_kb(connection)
 *
 * (&key (connection (local-connection)))
 *
 * Return Value(s): meta-kb 
 * 
 * Flags: O R 
 * 
 * Returns the Meta-KB for the server accessed through the
 * connection.
 */

okbc_node *okbc_meta_kb(const char *connection)
{
    okbc_node *ret = NULL;

    return ret;
}

/* okbc_next(enumerator)
 *
 * (enumerator)
 *
 * Return Value(s): value 
 * 
 * Flags: O R 
 * 
 * Return the next value for the enumerator. It is an error to
 * call next if has-more would return false for the
 * enumerator. If next is called on an enumerator that has been 
 * exhausted, an enumerator-exhausted error should be signaled.
 */

okbc_node *okbc_next(const char *enumerator)
{
    okbc_node *ret = NULL;

    return ret;
}

/* okbc_okbc_condition_p(thing)
 *
 * (thing)
 *
 * Return Value(s): boolean 
 * 
 * Flags: O R 
 * 
 * Returns true if thing is an OKBC-defined condition object, and
 * false otherwise.
 */

okbc_node *okbc_okbc_condition_p(const char *thing)
{
    okbc_node *ret = NULL;

    return ret;
}

/* okbc_openable_kbs(kb_type, connection, place)
 *
 * (&key kb-type (connection (local-connection)) place)
 *
 * Return Value(s): list-of-kb-locators 
 * 
 * Flags: M R 
 * 
 * Given a kb-type and a connection, returns
 * list-of-kb-locators, a list of frame handles to
 * frames in the meta-kb that are instances of the class 
 * kb-locator.
 * Each kb-locator instance describes one openable KB, identified at the
 * time of the call. Subsequent calls to openable-kbs will refresh the
 * set of kb-locator instances in the meta-kb. Kb-locators refering to
 * KBs of kb-type that are no longer openable will be removed. KBs
 * of kb-type that have become openable since the last call will
 * become represented in the meta-kb.
 *
 * Place allows the application to communicate to the KRS in an
 * KRS-specific way where to look for the openable KBs (e.g., a directory).
 * The use of the place argument is not portable.
 * If a particular kb-type does not understand the value of
 * the place argument supplied, openable-kbs returns
 * (), that is, there are no known openable KBs consistent
 * with the supplied place.
 */

okbc_node *okbc_openable_kbs(const char *kb_type, const char *connection,
			     const char *place)
{
    okbc_node *ret = NULL;

    return ret;
}

/* okbc_open_kb(kb_locator, kb_type, connection, error_p)
 *
 * (kb-locator &key kb-type (connection (local-connection)) (error-p true))
 *
 * Return Value(s): kb 
 * 
 * Flags: M W 
 * 
 * Given a kb-locator, a kb-type, and a
 * connection, returns a KB object
 * for that KB locator that will behave as if all the objects in the
 * KB are accessible (the implementation is not actually required to load the
 * whole KB into memory).
 *
 * Implementations are at liberty to accept other values in place of the
 * kb-locator, such as a pathname that identifies the location of
 * the KB to the system. Such usage is convenient, but is not portable. It
 * is not portable for an OKBC application to use anything other than
 * a KB locator for this argument. If error-p is false, catches
 * errors that occur, and attempts to continue with the opening/loading
 * process. If the KB could not be successfully opened, returns false.
 */

okbc_node *okbc_open_kb(const char *kb_locator, const char *kb_type,
			const char *connection, int error_p)
{
    okbc_node *ret = NULL;

    return ret;
}

/* okbc_prefetch(enumerator, number_of_values, increment)
 *
 * (enumerator &key (number-of-values :all) increment)
 *
 * Return Value(s): void 
 * 
 * Flags: O R 
 * 
 * The prefetch operator is an important optimization in network
 * settings. The client will attempt to prefetch sufficient elements in
 * enumerator from the server so that number-of-values elements
 * will be immediately available (cached) at the enumerator, using
 * only a single network call rather than executing network calls for each 
 * element.
 *
 * If it is discovered that there are fewer than number-of-values
 * elements cached locally by enumerator, a minimum chunk of
 * increment elements will be prefetched, when available. Thus, if
 * the enumerator already holds five elements locally, and the call
 * (prefetch enumerator 7 20)
 * is made, the fact that seven elements are requested, but only five are 
 * available means that a request will in fact be made to the server for more
 * elements, and at least another 20 (as opposed to 2) elements will be 
 * prefetched. When increment is false, the number of elements 
 * prefetched will be the difference between the number currently cached in 
 * the enumerator and number-of-values.
 *
 * Note that unlike other operations taking a number-of-values 
 * argument, this operation does not return a more-status value. 
 * Returns no values.
 */

void okbc_prefetch(const char *enumerator, int number_of_values,
		   const char *increment)
{
}

/* okbc_primitive_p(class, kb, kb_local_only_p)
 *
 * (class &key kb kb-local-only-p)
 *
 * Return Value(s): boolean 
 * 
 * Flags: M R 
 * 
 * Returns true iff class is a class whose definition is
 * primitive, and false otherwise. For KRSs that do not distinguish
 * primitive from defined classes, primitive-p
 * must return true for all classes.
 */

okbc_node *okbc_primitive_p(const char *class, const char *kb,
			    int kb_local_only_p)
{
    okbc_node *ret = NULL;

    return ret;
}

/* okbc_print_frame(frame, kb, slots, facets, stream, inference_level, number_of_values, value_selector, kb_local_only_p)
 *
 * (frame &key kb (slots :filled) (facets :filled) (stream true) (inference-level :taxonomic) (number-of-values :all) (value-selector :either) kb-local-only-p)
 *
 * Return Value(s): false-or-string 
 * 
 * Flags: O R 
 * 
 * Prints a textual representation to stream of the
 * frame. A warning is printed when frame is not
 * coercible to a frame. Stream objects as values to the stream
 * argument are generally system- and implementation-specific, so stream
 * objects will not in general be
 * transmissible in networked implementations. Two special values for the
 * stream argument are also supported. If stream is false,
 * then print-frame prints to a string stream and
 * that string is returned as false-or-string.
 * If stream is true, print-frame attempts to print to the
 * standard output stream of the server on which print-frame
 * runs. When kb is
 * a network-acessed KB, this latter option is unlikely to have a useful
 * effect. Unless stream is false, false-or-string is
 * false.
 *
 * The slots and facets arguments control which slots (facets) 
 * are to be displayed. They can each take on one of the following values:
 * 
 * :all - Shows all applicable slots (facets)
 * :none - Shows no slots (facets)
 * :filled - Shows the subset of slots (facets) that have at
 * least one value for frame
 * list of slots (facets) - Only the listed slots (facets) are shown,
 * if present
 */

okbc_node *okbc_print_frame(const char *frame, const char *kb,
			    const char *slots, const char *facets,
			    const char *stream,
			    const char *inference_level,
			    int number_of_values,
			    const char *value_selector,
			    int kb_local_only_p)
{
    okbc_node *ret = NULL;

    return ret;
}

/* okbc_procedure_p(thing)
 *
 * (thing)
 *
 * Return Value(s): boolean 
 * 
 * Flags: R 
 * 
 * Is true if thing is a procedure, and false otherwise.
 */

okbc_node *okbc_procedure_p(const char *thing)
{
    okbc_node *ret = NULL;

    return ret;
}

/* okbc_register_procedure(name, procedure, kb)
 *
 * (name procedure &key kb)
 *
 * Return Value(s): void 
 * 
 * Flags: O W 
 * 
 * Associates the procedure with name in kb.
 * Subsequent calls to call-procedure may invoke the procedure merely by
 * using the name. If name is false, then procedure
 * must be a named procedure, in which case the name
 * argument will default to the name of the procedure.
 * Returns no values.
 */

void okbc_register_procedure(const char *name, const char *procedure,
			     const char *kb)
{
}

/* okbc_revert_kb(kb, error_p)
 *
 * (&key kb (error-p true))
 *
 * Return Value(s): reverted-kb 
 * 
 * Flags: O W 
 * 
 * This operation is called when the user wishes to discard any changes
 * made to a KB since it was last saved, and revert to the previously saved
 * state.
 *
 * Equivalent to successive calls to close-kb and then open-kb.
 * The reverted-kb returned has the same content as kb had at
 * the time it was last saved (or empty, if the kb had never
 * been saved). No guarantee is made that the reverted-kb
 * will have the same identity (==, EQLness) as kb,
 * but some implementations may choose to recycle the existing KB object.
 * After a call to revert-kb, portable applications must refer only
 * to the reverted-kb object rather than kb in case it was
 * not recycled. References to the original kb may result in
 * an object-freed condition being signaled.
 * If error-p is false, tries to catch errors that occur, and
 * attempts to continue with reverting to the extent possible.
 */

okbc_node *okbc_revert_kb(const char *kb, int error_p)
{
    okbc_node *ret = NULL;

    return ret;
}

/* okbc_subclass_of_p(subclass, superclass, kb, inference_level, kb_local_only_p)
 *
 * (subclass superclass &key kb (inference-level :taxonomic) kb-local-only-p)
 *
 * Return Value(s): boolean exact-p 
 * 
 * Flags: O R 
 * 
 * Returns true if class subclass is a subclass of class 
 * superclass, and returns false otherwise.
 */

okbc_node *okbc_subclass_of_p(const char *subclass, const char *superclass,
			      const char *kb, const char *inference_level,
			      int kb_local_only_p)
{
    okbc_node *ret = NULL;

    return ret;
}

/* okbc_superclass_of_p(superclass, subclass, kb, inference_level, kb_local_only_p)
 *
 * (superclass subclass &key kb (inference-level :taxonomic) kb-local-only-p)
 *
 * Return Value(s): boolean exact-p 
 * 
 * Flags: O R 
 * 
 * Returns true if class subclass is a subclass of class
 * superclass, and returns false otherwise.
 */

okbc_node *okbc_superclass_of_p(const char *superclass,
				const char *subclass, const char *kb,
				const char *inference_level,
				int kb_local_only_p)
{
    okbc_node *ret = NULL;

    return ret;
}

/* okbc_tellable(sentence, kb, value_selector, kb_local_only_p)
 *
 * (sentence &key kb (value-selector :known-true) kb-local-only-p)
 *
 * Return Value(s): boolean 
 * 
 * Flags: O R 
 * 
 * The tellable operation returns false if the KRS can
 * determine that telling the sentence would result in a
 * cannot-handle error being signaled, and true otherwise.
 * It may raise a syntax-error error as specified with the
 * definition of tell. Even if tellable returns
 * true, tell may still not be able to handle the
 * sentence. 
 */

okbc_node *okbc_tellable(const char *sentence, const char *kb,
			 const char *value_selector, int kb_local_only_p)
{
    okbc_node *ret = NULL;

    return ret;
}

/* okbc_tell(sentence, kb, frame, value_selector, kb_local_only_p)
 *
 * (sentence &key kb frame (value-selector :known-true) kb-local-only-p)
 *
 * Return Value(s): void 
 * 
 * Flags: O R 
 * 
 * The tell operation asserts the sentence to be true in
 * the knowledge base kb. Some KRSs may allow users to attach a
 * sentence to a specific frame in the KB. If that is possible and desired, 
 * the frame argument may be supplied. When kb-local-only-p
 * is true, the sentence should be asserted in the kb, even
 * if kb includes another KB containing the sentence. When the
 * sentence argument is syntactically invalid, tell signals
 * a syntax-error error. An KRS may not accept all valid sentences
 * of the OKBC Assertion Language, and for such cases, tell raises the
 * condition cannot-handle. Returns no values.
 */

void okbc_tell(const char *sentence, const char *kb, const char *frame,
	       const char *value_selector, int kb_local_only_p)
{
}

/* okbc_type_of_p(class, thing, kb, inference_level, kb_local_only_p)
 *
 * (class thing &key kb (inference-level :taxonomic) kb-local-only-p)
 *
 * Return Value(s): boolean exact-p 
 * 
 * Flags: O R 
 * 
 * Returns true if thing is an instance of class, otherwise
 * returns false.
 */

okbc_node *okbc_type_of_p(const char *class, const char *thing,
			  const char *kb, const char *inference_level,
			  int kb_local_only_p)
{
    okbc_node *ret = NULL;

    return ret;
}

/* okbc_unregister_procedure(name, kb)
 *
 * (name &key kb)
 *
 * Return Value(s): void 
 * 
 * Flags: O W 
 * 
 * Removes any procedure association for the name in kb.
 * Returns no values.
 */

void okbc_unregister_procedure(const char *name, const char *kb)
{
}

/* okbc_untell(sentence, kb, frame, value_selector, kb_local_only_p)
 *
 * (sentence &key kb frame (value-selector :known-true) kb-local-only-p)
 *
 * Return Value(s): boolean 
 * 
 * Flags: O R 
 * 
 * The untell operation can be used to remove assertions from the
 * knowledge base. Returns true if the sentence was removed, and 
 * false otherwise.
 *
 * The effect of untelling a sentence of the OKBC Assertion Language
 * is equivalent to the effect of executing an equivalent OKBC operation. 
 * The OKBC operations equivalent to untelling a sentence in the
 * Assertion Language are specified in Section 3.5.7.
 * For example, the operation
 * 
 * (Untell `(slot-name frame-name ,slot-value))
 * is equivalent to the OKBC operation (remove-slot-value 'frame-name
 * 'slot-name slot-value :slot-type :own).
 *
 * The effect of untelling an arbitrary tellable sentence
 * is not predictable across implementations. For a given sentence s, executing (untell s :kb kb) after executing (tell s :kb
 * kb) should remove the sentence s from the kb. That is, a
 * second call to untell should return nil. This
 * does not mean that s is no longer true, as it may be implied by
 * other assertions in the KB.
 *
 * Some KRSs may allow users to attach an assertion to a frame in the KB.
 * If the sentence was attached to a frame, the frame
 * argument must be supplied. The default value for frame is
 * false. When the sentence argument is syntactically invalid, it
 * may signal the syntax-error error.
 */

okbc_node *okbc_untell(const char *sentence, const char *kb,
		       const char *frame, const char *value_selector,
		       int kb_local_only_p)
{
    okbc_node *ret = NULL;

    return ret;
}

/* okbc_value_as_string(value, kb, purpose, pretty_p, kb_local_only_p)
 *
 * (value &key kb purpose pretty-p kb-local-only-p)
 *
 * Return Value(s): string location-list 
 * 
 * Flags: O R 
 * 
 * Given some value, returns two values, a string that is a printed
 * representation of that value, and a list of three-tuples.
 * In the second location-list value, one tuple is supplied for each
 * frame reference encountered in value in the order they appear
 * in the printed representation. Each three-tuple is of the form
 * (index0 index1 frame), where index0 is the zero-indexed
 * start index of a printed representation of the
 * frame in the printed representation and index1 is the
 * index one past the end. For example, printing the list (#&lt;frame1&gt;)
 * might deliver the values "(Frame1)" and ((1 8 #&lt;frame1&gt;)).
 *
 * This operation is useful for user interfaces that have no way in
 * general to print the arbitrary data structures that might be returned by,
 * for example, get-slot-values. The second value allows user
 * interfaces to print the string with mouse-sensitive regions that point to
 * frames.
 * 
 * Purpose
 * - The purpose argument can be one of {:file,
 * :user-interface}. When it is :file, it allows the
 * KB to print data in a manner that will be readable again (e.g.,
 * for dumping to files),
 * preserving object identity where necessary. The output will
 * therefore typically fully escape strings, and will probably
 * package qualify symbols. When purpose is
 * :user-interface, the string returned will be as pretty
 * and readable as possible for a user interface, but such
 * output will probably not be readable by OKBC.
 * 
 * Pretty-p
 * - When true, OKBC will attempt to print everything in as pretty a
 * manner as possible. This includes printing frames by using their
 * pretty-names, and by printing strings without escaping or
 * quotes.
 */

okbc_node *okbc_value_as_string(const char *value, const char *kb,
				const char *purpose, int pretty_p,
				int kb_local_only_p)
{
    okbc_node *ret = NULL;

    return ret;
}

/* okbc_attach_facet(frame, slot, facet, kb, slot_type, kb_local_only_p)
 *
 * (frame slot facet &key kb (slot-type :own) kb-local-only-p)
 *
 * Return Value(s): void 
 * 
 * Flags: M W 
 * 
 * Explicitly associates the facet with slot on frame,
 * in the sense of recording that values of the facet may be asserted with
 * frame or with instances of frame if slot-type is
 * :template.
 * As a result, facet is returned by get-slot-facets at the
 * :direct inference level, and slot-has-facet-p will be
 * true for facet in slot on frame. It is an
 * error to attempt to attach a non-existent facet. Doing so should signal
 * a facet-not-found error. Returns no values.
 */

void okbc_attach_facet(const char *frame, const char *slot,
		       const char *facet, const char *kb,
		       const char *slot_type, int kb_local_only_p)
{
}

/* okbc_attach_slot(frame, slot, kb, slot_type, kb_local_only_p)
 *
 * (frame slot &key kb (slot-type :own) kb-local-only-p)
 *
 * Return Value(s): void 
 * 
 * Flags: M W 
 * 
 * Explicitly associates the slot with frame, in the sense
 * of recording that values of slot may be asserted with frame or
 * with instances of frame if slot-type is :template.
 * As a result, slot is returned by get-frame-slots at the
 * :direct inference level, and frame-has-slot-p will be
 * true for slot on frame. It is an error to attempt to
 * attach a non-existent slot. Doing so should signal a
 * slot-not-found error. Returns no values.
 */

void okbc_attach_slot(const char *frame, const char *slot, const char *kb,
		      const char *slot_type, int kb_local_only_p)
{
}

/* okbc_close_connection(connection, force_p, error_p)
 *
 * (connection &key force-p (error-p true))
 *
 * Return Value(s): void 
 * 
 * Flags: O R 
 * 
 * Closes the connection. If force-p is true, the connection
 * may be closed without waiting for any handshakes from the server. A call
 * to close-connection on the local connection
 * is a no-op. This allows the user to loop through all-connections,
 * closing them all without fear of losing connectivity to KBs that share the
 * same address space as the application. Returns no values.
 */

void okbc_close_connection(const char *connection, int force_p,
			   int error_p)
{
}

/* okbc_close_kb(kb, save_p)
 *
 * (&key kb save-p)
 *
 * Return Value(s): void 
 * 
 * Flags: M W 
 * 
 * Deletes any in-core/accessible representation of kb, but does not
 * remove it from any persistent store if the persistent version still
 * constitutes a meaningful KB (i.e., temporary disk work files would be
 * deleted). It is an error ever to use kb again for any purpose. If
 * this occurs, an object-freed error should be signaled.
 * Implementations may free any storage allocated for KB. If save-p
 * is true, then any unsaved changes to kb will be saved
 * before it is closed. Note that the default value of save-p
 * is false. Returns no values.
 */

void okbc_close_kb(const char *kb, int save_p)
{
}

/* okbc_copy_frame(frame, new_name, kb, to_kb, error_p, missing_frame_action, frame_handle_mapping_table, kb_local_only_p)
 *
 * (frame new-name &key kb (to-kb (current-kb)) (error-p true) (missing-frame-action :stop) frame-handle-mapping-table kb-local-only-p)
 *
 * Return Value(s): copy-of-frame allocated-frame-handle-alist 
 * 
 * Flags: O W 
 * 
 * Copies frame from kb to to-kb. The name of the new
 * frame in to-kb will be new-name. Kb and
 * to-kb may be the same KB. If the :frame-names-required
 * behavior has the value false for kb, new-name may be
 * false.
 
 * If error-p is false, catches errors that occur, and 
 * continues copying to the extent possible.
 *
 * The frame may contain references to other frames that do not
 * reside in to-kb - for example,
 * its types, superclasses, or slot values.
 * Missing-frame-action controls the behavior of copy-frame 
 * in this case. It can take one of the following values:
 *
 * :stop - Stop copying and signal a frames-missing error,
 * depending on the value of error-p.
 *
 * :abort - Abort copying frame, leaving the state of
 * to-kb unchanged. Any side effects of
 * copy-frame that have already been performed will
 * be undone. Signals a frames-missing error,
 * depending on the value of error-p.
 *
 * :allocate - Allocate frame handles for any frame references that 
 * do not yet exist in to-kb.
 *
 * :ignore - Continue with the copying of the current frame, but
 * ignore and remove any references to missing frames.
 *
 * Frame-handle-mapping-table, if supplied, is a hash table that maps
 * the frame handles in the kb to frame handles in to-kb, and
 * is used during compound copy operations, such as copy-kb. If
 * copy-frame fails to find a referenced frame in to-kb, it looks up
 * the reference in the Frame-handle-mapping-table before allocating 
 * a new frame handle.
 *
 * It returns two values.
 * 
 * 1.
 * Copy-of-frame - Identifies the newly created frame in
 * to-kb. If copy-frame terminates because some frames were
 * missing, and missing-frame-action was :abort,
 * false is returned as a value of copy-of-frame.
 * 2.
 * Allocated-frame-handle-alist - a list of 2-tuples
 * (frame-handle-in-kb frame-handle-in-to-kb) that maps frame handles in
 * kb to frame handles in to-kb that were allocated during
 * the copying process. These mappings will also have been entered in
 * frame-handle-mapping-table if it was supplied.
 */

okbc_node *okbc_copy_frame(const char *frame, const char *new_name,
			   const char *kb, const char *to_kb, int error_p,
			   const char *missing_frame_action,
			   const char *frame_handle_mapping_table,
			   int kb_local_only_p)
{
    okbc_node *ret = NULL;

    return ret;
}

/* okbc_copy_kb(from_kb, to_kb, error_p, missing_frame_action, kb_local_only_p)
 *
 * (from-kb to-kb &key (error-p true) (missing-frame-action :stop) kb-local-only-p)
 *
 * Return Value(s): void 
 * 
 * Flags: O W 
 * 
 * Copies the frames in from-kb into to-kb.
 * The interpretation of Missing-frame-action is the same as
 * for copy-frame. If error-p is false, catches errors that
 * occur, and attempts to continue with copying. Returns no values.
 *
 * Note that the behavior are-frames might have different values for
 * the two KBs. Thus, if slots are represented as frames in
 * kb, but are not represented as frames in to-kb, the frames
 * representing slots in kb will not be copied.
 */

void okbc_copy_kb(const char *from_kb, const char *to_kb, int error_p,
		  const char *missing_frame_action, int kb_local_only_p)
{
}

/* okbc_detach_facet(frame, slot, facet, kb, slot_type, kb_local_only_p)
 *
 * (frame slot facet &key kb (slot-type :own) kb-local-only-p)
 *
 * Return Value(s): void 
 * 
 * Flags: M W 
 * 
 * Removes any explicit association between the facet and
 * slot on frame. As a result, facet is not returned
 * by get-slot-facets at inference-level :direct unless there are
 * facet values associated with facet in slot on
 * frame.
 */

void okbc_detach_facet(const char *frame, const char *slot,
		       const char *facet, const char *kb,
		       const char *slot_type, int kb_local_only_p)
{
}

/* okbc_detach_slot(frame, slot, kb, slot_type, kb_local_only_p)
 *
 * (frame slot &key kb (slot-type :own) kb-local-only-p)
 *
 * Return Value(s): void 
 * 
 * Flags: M W 
 * 
 * Removes any explicit association between the slot and
 * frame. As a result, slot is not returned by
 * get-frame-slots at inference-level :direct unless there are
 * slot or facet values associated with slot in frame.
 */

void okbc_detach_slot(const char *frame, const char *slot, const char *kb,
		      const char *slot_type, int kb_local_only_p)
{
}

/* okbc_facet_has_value_p(frame, slot, facet, kb, inference_level, slot_type, value_selector, kb_local_only_p)
 *
 * (frame slot facet &key kb (inference-level :taxonomic) (slot-type :own) (value-selector :either) kb-local-only-p)
 *
 * Return Value(s): boolean exact-p 
 * 
 * Flags: O R 
 * 
 * Returns true iff the specified facet has a value for the specified slot
 * and frame, and otherwise returns false.
 */

okbc_node *okbc_facet_has_value_p(const char *frame, const char *slot,
				  const char *facet, const char *kb,
				  const char *inference_level,
				  const char *slot_type,
				  const char *value_selector,
				  int kb_local_only_p)
{
    okbc_node *ret = NULL;

    return ret;
}

/* okbc_facet_p(thing, kb, kb_local_only_p)
 *
 * (thing &key kb kb-local-only-p)
 *
 * Return Value(s): boolean 
 * 
 * Flags: M R 
 * 
 * Returns true iff thing is a facet, and false otherwise.
 */

okbc_node *okbc_facet_p(const char *thing, const char *kb,
			int kb_local_only_p)
{
    okbc_node *ret = NULL;

    return ret;
}

/* okbc_frame_has_slot_p(frame, slot, kb, inference_level, slot_type, kb_local_only_p)
 *
 * (frame slot &key kb (inference-level :taxonomic) (slot-type :own) kb-local-only-p)
 *
 * Return Value(s): boolean 
 * 
 * Flags: O R 
 * 
 * Returns true iff thing is a slot in frame,
 * otherwise returns false.
 */

okbc_node *okbc_frame_has_slot_p(const char *frame, const char *slot,
				 const char *kb,
				 const char *inference_level,
				 const char *slot_type,
				 int kb_local_only_p)
{
    okbc_node *ret = NULL;

    return ret;
}

/* okbc_frame_in_kb_p(thing, kb, kb_local_only_p)
 *
 * (thing &key kb kb-local-only-p)
 *
 * Return Value(s): boolean 
 * 
 * Flags: O R 
 * 
 * Returns true when thing is both coercible to
 * a frame, and that frame is known to be resident in kb, and otherwise
 * returns false. See get-frame-in-kb.
 */

okbc_node *okbc_frame_in_kb_p(const char *thing, const char *kb,
			      int kb_local_only_p)
{
    okbc_node *ret = NULL;

    return ret;
}

/* okbc_frs_independent_frame_handle(frame, kb, kb_local_only_p)
 *
 * (frame &key kb kb-local-only-p)
 *
 * Return Value(s): frame-handle 
 * 
 * Flags: O R 
 * 
 * Given a frame, returns frame-handle, which is a KRS-independent
 * OKBC frame handle object. Frame-handle may now be used in network
 * applications to refer to frame or in communication between KBs.
 * The correspondence between frame and frame-handle is
 * maintained, so that subsequent calls with the same frame will return the
 * same frame-handle.
 *
 * It is not anticipated that this operation will ever be called by user
 * applications, but must be used by back ends to implement
 * decontextualize.
 *
 * Note: This operation is named frs-independent-frame-handle for
 * historical reasons. Frame Representation Systems are now uniformly
 * called Knowledge Representation Systems with the exception of in the names
 * of this operator and frs-name.
 */

okbc_node *okbc_frs_independent_frame_handle(const char *frame,
					     const char *kb,
					     int kb_local_only_p)
{
    okbc_node *ret = NULL;

    return ret;
}

/* okbc_frs_name(kb_type, connection)
 *
 * (&key kb-type (connection (local-connection)))
 *
 * Return Value(s): krs-name 
 * 
 * Flags: O R 
 * 
 * Returns the krs-name of the underlying KRS associated with the
 * kb-type, which is accessed over connection.
 * Krs-name is a string. For example,
 * given loom-kb as the kb-type, it might return the string
 * "LOOM". This operation is used by user interfaces that need to
 * display a printed representation of the underlying KRS for a particular
 * kb-type.
 *
 * Note: This operation is named frs-name for
 * historical reasons. Frame Representation Systems are now uniformly
 * called Knowledge Representation Systems with the exception of in the names
 * of this operator and frs-independent-frame-handle.
 */

okbc_node *okbc_frs_name(const char *kb_type, const char *connection)
{
    okbc_node *ret = NULL;

    return ret;
}

/* okbc_kb_modified_p(kb)
 *
 * (&key kb)
 *
 * Return Value(s): boolean 
 * 
 * Flags: O R 
 * 
 * Returns true if kb has been modified since it was last saved.
 */

okbc_node *okbc_kb_modified_p(const char *kb)
{
    okbc_node *ret = NULL;

    return ret;
}

/* okbc_kb_p(thing)
 *
 * (thing)
 *
 * Return Value(s): boolean 
 * 
 * Flags: O R 
 * 
 * Returns true if thing is a KB, otherwise returns false.
 */

okbc_node *okbc_kb_p(const char *thing)
{
    okbc_node *ret = NULL;

    return ret;
}

/* okbc_rename_facet(facet, new_name, kb, kb_local_only_p)
 *
 * (facet new-name &key kb kb-local-only-p)
 *
 * Return Value(s): renamed-facet 
 * 
 * Flags: M W 
 * 
 * Renames the facet for all frames containing that facet.
 * 
 * If the facet identified by facet is represented as a frame,
 * that frame is renamed.
 * If the facet identified by facet is not represented as
 * a frame, facet-p applied to facet will return
 * false and facet will not be returned by any of
 * the facet-returning operations, such as get-kb-facets.
 * New-name will now identify the
 * facet, and will be returned by operations such as
 * get-kb-facets and get-frame-facets.
 * All the facet values and facet values associated with facet are
 * preserved under the new-name. For example, for any frame in
 * the KB, the values returned by get-facet-values for facet
 * before the rename are identical to the values returned for
 * new-name after the rename. In addition, the attached-to
 * relationship is preserved - that is, if facet is attached to a
 * frame before the rename, new-name is attached to that frame
 * after the rename. In some implementations, references to facet
 * may still remain in the KB after rename-facet. Returns the
 * renamed-facet.
 */

okbc_node *okbc_rename_facet(const char *facet, const char *new_name,
			     const char *kb, int kb_local_only_p)
{
    okbc_node *ret = NULL;

    return ret;
}

/* okbc_rename_slot(slot, new_name, kb, kb_local_only_p)
 *
 * (slot new-name &key kb kb-local-only-p)
 *
 * Return Value(s): renamed-slot 
 * 
 * Flags: M W 
 * 
 * Renames the slot for all frames containing that slot.
 * 
 * If the slot identified by slot is represented as a frame,
 * that frame is renamed.
 * If the slot identified by slot is not represented as
 * a frame, slot-p applied to slot will return false
 * and slot will not be returned by any of the slot-returning
 * operations, such as get-kb-slots and get-frame-slots.
 * New-name will now identify the slot, and will be returned by
 * operations such as get-kb-slots and get-frame-slots.
 * All the slot values and facet values associated with slot are
 * preserved under the new-name. For example, for any frame in
 * the KB, the values returned by get-slot-values for slot
 * before the rename are identical to the values returned for
 * new-name after the rename. In addition, the attached-to
 * relationship is preserved - that is, if slot is attached to a
 * frame before the rename, new-name is attached to that frame
 * after the rename. In some implementations, references to slot
 * may still remain in the KB after rename-slot. Returns the
 * renamed-slot.
 */

okbc_node *okbc_rename_slot(const char *slot, const char *new_name,
			    const char *kb, int kb_local_only_p)
{
    okbc_node *ret = NULL;

    return ret;
}

/* okbc_replace_facet_value(frame, slot, facet, old_value, new_value, kb, test, slot_type, value_selector, kb_local_only_p)
 *
 * (frame slot facet old-value new-value &key kb (test :equal) (slot-type :own) (value-selector :known-true) kb-local-only-p)
 *
 * Return Value(s): void 
 * 
 * Flags: O W 
 * 
 * If old-value is currently a member of the set of direct values of
 * the specified facet, then old-value is replaced by 
 * new-value in the facet. Returns no values.
 */

void okbc_replace_facet_value(const char *frame, const char *slot,
			      const char *facet, const char *old_value,
			      const char *new_value, const char *kb,
			      const char *test, const char *slot_type,
			      const char *value_selector,
			      int kb_local_only_p)
{
}

/* okbc_replace_slot_value(frame, slot, old_value, new_value, kb, test, slot_type, index, value_selector, kb_local_only_p)
 *
 * (frame slot old-value new-value &key kb (test :equal) (slot-type :own) (index :all) (value-selector :known-true) kb-local-only-p)
 *
 * Return Value(s): void 
 * 
 * Flags: O W 
 * 
 * If old-value is currently a member of the set of direct values of
 * slot, then old-value is replaced by new-value in
 * slot. If index is :all then all occurrences of
 * old-value will be replaced. Otherwise, index should be
 * an integer index into the values list, and only the value at that
 * position, if it matches old-value, will be replaced (the first
 * value of the slot has index 0).
 * This operation may signal constraint violation conditions
 * (see Section 3.8). The index argument is used only
 * by slots whose :collection-type is :list.
 * Returns no values.
 */

void okbc_replace_slot_value(const char *frame, const char *slot,
			     const char *old_value, const char *new_value,
			     const char *kb, const char *test,
			     const char *slot_type, const char *index,
			     const char *value_selector,
			     int kb_local_only_p)
{
}

/* okbc_save_kb(kb, error_p)
 *
 * (&key kb (error-p true))
 *
 * Return Value(s): boolean 
 * 
 * Flags: M W 
 * 
 * Saves the contents of the KB to persistent storage. No commitment is
 * made as to the location of the KB in persistent storage, other than that
 * it will be openable given the name, kb-type and connection first used
 * to access it. No commitment is made as to whether the save operation
 * results in a complete dump of the KB, or whether it results only in a
 * dump of the changes made since the KB was last saved. If error-p
 * is false, tries to catch errors that occur, and attempts to continue 
 * with saving to the extent possible.
 * Returns true iff the KB was saved successfully, and false otherwise.
 */

okbc_node *okbc_save_kb(const char *kb, int error_p)
{
    okbc_node *ret = NULL;

    return ret;
}

/* okbc_save_kb_as(new_name_or_locator, kb)
 *
 * (new-name-or-locator &key kb)
 *
 * Return Value(s): void 
 * 
 * Flags: M W 
 * 
 * Saves the entire contents of the KB to persistent storage under the 
 * new-name-or-locator. New-name-or-locator is either a new
 * name for the KB, or a new kb-locator. The in-core KB is renamed so that
 * find-kb-of-type will return kb when passed the new name of
 * the KB. Returns no values.
 */

void okbc_save_kb_as(const char *new_name_or_locator, const char *kb)
{
}
