/*
 *  Copyright (C) 2002 Steve Harris, ECS, University of Southampton
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  $Id: okbc_put.c,v 1.2 2003/02/20 19:01:18 theno23 Exp $
 */

#include <stdio.h>

#include "rdfsql/rdfsql.h"
#include "okbc_types.h"

/* okbc_put_behavior_values(behavior, values, kb)
 *
 * (behavior values &key kb)
 *
 * Return Value(s): void 
 * 
 * Flags: M R 
 * 
 * Sets the list of active values of the behavior under
 * which the KB is to operate. The elements in values must be a
 * subset of the values returned by a call to
 * get-behavior-supported-values for the same behavior.
 * If they are not, an illegal-behavior-values error will be signaled.
 * Note that for some behaviors, the order of values is significant (e.g.,
 * :collection-type). Returns no values.
 */

void okbc_put_behavior_values(const char *behavior, const char *values,
			      const char *kb)
{
}

/* okbc_put_class_superclasses(class, new_superclasses, kb, kb_local_only_p)
 *
 * (class new-superclasses &key kb kb-local-only-p)
 *
 * Return Value(s): void 
 * 
 * Flags: M W 
 * 
 * Changes class to be a subclass of all the classes listed in
 * new-superclasses. If frame was a subclass of any
 * superclasses not mentioned in new-superclasses, these superclasses
 * are removed.
 * This operation may signal constraint violation conditions
 * (see Section 3.8). Returns no values.
 */

void okbc_put_class_superclasses(const char *class,
				 const char *new_superclasses,
				 const char *kb, int kb_local_only_p)
{
}

/* okbc_put_facet_value(frame, slot, facet, value, kb, slot_type, value_selector, kb_local_only_p)
 *
 * (frame slot facet value &key kb (slot-type :own) (value-selector :known-true) kb-local-only-p)
 *
 * Return Value(s): void 
 * 
 * Flags: O W 
 * 
 * Sets the values of the specified facet to be a singleton set
 * consisting of a single element: value. Returns no values.
 */

void okbc_put_facet_value(const char *frame, const char *slot,
			  const char *facet, const char *value,
			  const char *kb, const char *slot_type,
			  const char *value_selector, int kb_local_only_p)
{
}

/* okbc_put_facet_values(frame, slot, facet, values, kb, slot_type, value_selector, kb_local_only_p)
 *
 * (frame slot facet values &key kb (slot-type :own) (value-selector :known-true) kb-local-only-p)
 *
 * Return Value(s): void 
 * 
 * Flags: M W 
 * 
 * Sets the values of the specified facet to be
 * values, which is assumed to be a set. Any existing facet values
 * that are not in values will be removed. The order of the elements of
 * values will not necessarily be maintained by the KRS.
 * This operation may signal constraint violation conditions
 * (see Section 3.8). Returns no values.
 */

void okbc_put_facet_values(const char *frame, const char *slot,
			   const char *facet, const char *values,
			   const char *kb, const char *slot_type,
			   const char *value_selector, int kb_local_only_p)
{
}

/* okbc_put_frame_details(frame, details, kb, kb_local_only_p)
 *
 * (frame details &key kb kb-local-only-p)
 *
 * Return Value(s): void 
 * 
 * Flags: O W 
 * 
 * Redefines frame to have the specified details.
 * Details is a property list as specified for
 * get-frame-details. This operation is useful for systems that allow
 * transaction-oriented editing of multiple aspects of a frame.
 * The properties :handle, :frame-type, and :primitive-p
 * are ignored, since these may not be put.
 * Returns no values.
 */

void okbc_put_frame_details(const char *frame, const char *details,
			    const char *kb, int kb_local_only_p)
{
}

/* okbc_put_frame_name(frame, new_name, kb, kb_local_only_p)
 *
 * (frame new-name &key kb kb-local-only-p)
 *
 * Return Value(s): renamed-frame 
 * 
 * Flags: M W 
 * 
 * Changes the name of frame to be new-name. All references
 * to frame in kb (e.g., in slot values) will point to the
 * frame named new-name. Returns the frame with the new name,
 * renamed-frame. It is not necessary that the frame object identified
 * by frame be identical (==/EQLness) to the frame
 * object called new-name, only that the KB consistently use the
 * new frame instead of the old one.
 * 
 * Implementation note: KRSs that use frame names as frame handles must
 * replace all references to the old name of frame with
 * new-name. This specification allows for implementations that
 * are forced to replace the representation of the frame with a new, renamed
 * version.
 */

okbc_node *okbc_put_frame_name(const char *frame, const char *new_name,
			       const char *kb, int kb_local_only_p)
{
    okbc_node *ret = NULL;

    return ret;
}

/* okbc_put_frame_pretty_name(frame, name, kb, kb_local_only_p)
 *
 * (frame name &key kb kb-local-only-p)
 *
 * Return Value(s): void 
 * 
 * Flags: M W 
 * 
 * Stores the name, a string, as the new pretty-name of the
 * frame. OKBC mandates no constraints on the new pretty-name,
 * but to maximize the liklihood that applications will interoperate smoothly,
 * implementations are encouraged to make pretty-names be short, and are
 * strongly encouraged to include no whitespace characters other than the space
 * characters, or any display device-specific control characters. Returns no
 * values.
 */

void okbc_put_frame_pretty_name(const char *frame, const char *name,
				const char *kb, int kb_local_only_p)
{
}

/* okbc_put_instance_types(frame, new_types, kb, kb_local_only_p)
 *
 * (frame new-types &key kb kb-local-only-p)
 *
 * Return Value(s): void 
 * 
 * Flags: M W 
 * 
 * Changes frame to be an instance of all the classes listed in
 * new-types. If frame was an instance of any types not
 * mentioned in new-types, these types are removed.
 * This operation may signal constraint violation conditions
 * (see Section 3.8). Returns no values.
 */

void okbc_put_instance_types(const char *frame, const char *new_types,
			     const char *kb, int kb_local_only_p)
{
}

/* okbc_put_slot_value(frame, slot, value, kb, slot_type, value_selector, kb_local_only_p)
 *
 * (frame slot value &key kb (slot-type :own) (value-selector :known-true) kb-local-only-p)
 *
 * Return Value(s): void 
 * 
 * Flags: O W 
 * 
 * Sets the values of slot in frame to be a singleton set
 * consisting of a single element: value.
 * This operation may signal constraint violation conditions
 * (see Section 3.8). Returns no values.
 */

void okbc_put_slot_value(const char *frame, const char *slot,
			 const char *value, const char *kb,
			 const char *slot_type, const char *value_selector,
			 int kb_local_only_p)
{
}

/* okbc_put_slot_values(frame, slot, values, kb, slot_type, value_selector, kb_local_only_p)
 *
 * (frame slot values &key kb (slot-type :own) (value-selector :known-true) kb-local-only-p)
 *
 * Return Value(s): void 
 * 
 * Flags: M W 
 * 
 * Sets the values of slot in frame to
 * be values. Any existing slot values that are not in values
 * will be removed. The order of the elements
 * of values will not necessarily be maintained by the KRS, unless the
 * :collection-type of the slot is :list.
 * This operation may signal constraint violation conditions
 * (see Section 3.8). Returns no values.
 */

void okbc_put_slot_values(const char *frame, const char *slot,
			  const char *values, const char *kb,
			  const char *slot_type,
			  const char *value_selector, int kb_local_only_p)
{
}
