/*
 *  Copyright (C) 2002 Steve Harris, ECS, University of Southampton
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 */

#ifndef RDQL_DATATYPES_H
#define RDQL_DATATYPES_H

#include "rdfsql_datatypes.h"

typedef struct {
	const char  *name;
	rs_obj_type  type;
	const char  *value;
	const char  *column;
	const char  *val_column;
	const char  *hash_column;
	int          triple;
	unsigned int returned;
	unsigned int need_join;
} rdql_variable;

typedef struct {
	const char *variable;
	rs_obj_type type;
	const char *value;
} rs_rdql_column;

typedef struct {
	void *db_res;
	int columns;
	rs_rdql_column *row;
	char *sql_query;
	char *error_str;
	struct _query_t *query;
} rs_rdql_result;

typedef struct {
	int    length;
	int    size;
	char **vec;
} strvec_t;

typedef struct {
	int         type;
	const char *value;
} part_t;

typedef struct {
	part_t *s;
	part_t *p;
	part_t *o;
	part_t *m;
} pattern_t;

typedef struct {
	const char   *prefix;
	unsigned int prefix_len;
	const char   *uri;
} prefix_t;

typedef struct {
	part_t       *lval;
	unsigned int  op;
	part_t       *rval;
} constraint_t;

typedef struct _query_t {
	GSList *select;
	GSList *from;
	GSList *where;
	GSList *prefixes;
	GSList *constr;
	int	limit;
	int 	offset;
} query_t;

rdql_variable *new_rdql_variable(const char *name);
char *rdql_variable_type_pp(rdql_variable *v);

part_t *new_part(int type, const char *value);
void expand_part(part_t *p, GSList *prefs);

prefix_t *new_prefix(const char *prefix, const char *uri);

constraint_t *new_constraint(part_t *lval, unsigned int op, part_t *r_val);

rdql_variable *constrain_var_type(GHashTable *st, const char *name,
                rs_obj_type type);

rdql_variable *require_var_join(GHashTable *st, const char *name,
                int need_join);

/* new_strvec(): creates a new string vector object for holding dynamically
 * sized arrays of strings */
strvec_t *new_strvec();

/* strvec_append(sv, s): appends s to the end of sv (constant time),
 * XXX NOT THREADSAFE */
void strvec_append(strvec_t *sv, const char *s);

/* returns the number of elements in the vector */
int strvec_length(strvec_t *sv);

/* strvec_join(sv, inter): returns all the elelemtents of sv, interspersed
 * with inter as a char *. Return value must be freed */
char *strvec_join(strvec_t *sv, const char *inter);

/* strvec_free(sv): frees sv and any memory it has allocated */
void strvec_free(strvec_t *sv);

void query_t_free(query_t *q);

#endif
