%{

#include <string.h>

#define _LEX_C_
#include "lex.h"
#include "rdfsql/rdql_datatypes.h"
#include "rdql_parser.h"

void uri();
void literal();
void variable();
void symbol();
void regex();
void eat_linecomment();
void eat_blockcomment();

%}

%%

"("       	{ return OPEN_PAR; }
")"       	{ return CLOSE_PAR; }

"SELECT"    	{ return SELECT; }
"select"    	{ return SELECT; }
"FROM"     	{ return FROM; }
"from"     	{ return FROM; }
"WHERE"    	{ return WHERE; }
"where"    	{ return WHERE; }
"USING"    	{ return USING; }
"using"    	{ return USING; }
"FOR"    	{ return FOR; }
"for"    	{ return FOR; }
"AND"    	{ return AND; }
"and"    	{ return AND; }
"LIMIT"    	{ return LIMIT; }
"limit"    	{ return LIMIT; }
"OFFSET"    	{ return OFFSET; }
"offset"    	{ return OFFSET; }

"eq"		{ return EQ_S; }
"ne"		{ return NE_S; }
"gt"		{ return GT_S; }
"lt"		{ return LT_S; }
"=="		{ return EQ_N; }
"="		{ return EQ_N; }
"!="		{ return NE_N; }
">"		{ return GT_N; }
"<"/[^A-Za-z=]	{ return LT_N; }
"LIKE"		{ return LIKE; }
"=~"		{ return EQ_RE; }

\?[a-zA-Z0-9]+ 	{ variable();        /* identifier: copy name */
          	  return VARIABLE; }
\<[^ >]*\> 	{ uri();             /* identifier: copy name */
          	  return URI; }
[a-zA-Z][a-zA-Z0-9]*:[^> ]*   {
                  symbol();
                  return URI; }

\"[^"]*\" 	{ literal();         /* identifier: copy name */
          	  return LITERAL; }
\'[^']*\' 	{ literal();         /* identifier: copy name */
          	  return LITERAL; }
\/[^\/]*\/  	{ regex();           /* regular expression: copy name */
          	  return REGEX; }
[+-]?[0-9]+	{ symbol();          /* number: copy string rep */
		  return INTEGER; }
[a-zA-Z][a-zA-Z0-9]* 	{
		  symbol();          /* identifier: copy name */
          	  return SYMBOL; }
"#"      	{ eat_linecomment(); }
"/*"      	{ eat_blockcomment(); }
","	  	{ return COMMA; }
\n        	{ lineno++; }
[ \t]+    	{}
;		{ return EOQ; }
.         	{ if (yytext[0] == EOF) {
			return EOQ;
		  }
		  printf("Unrecognised char: '%s' at line %d\n", yytext,
		  lineno); }

%%

// The comment-skipping function: skip to end-of-line
void eat_linecomment()  {
   char c;

   while ((c = input()) != '\n' && c != 0);
   lineno++;
}

void eat_blockcomment()  {
    char c, cm1 = '\0';

    while ((c = input())) {
	if (c == '\n') {
	    lineno++;
	}
	if (cm1 == '*' && c == '/') {
	    break;
	}
	cm1 = c;
    }
}

void uri()
{
   yylval.str = malloc(strlen(yytext) - 1);
   strncpy(yylval.str, yytext + 1, strlen(yytext) - 2);
   yylval.str[(strlen(yytext) - 2)] = '\0';
}

void literal()
{
   yylval.str = malloc(strlen(yytext) - 1);
   strncpy(yylval.str, yytext + 1, strlen(yytext) - 2);
   yylval.str[(strlen(yytext) - 2)] = '\0';
}

void variable()
{
   yylval.str = strdup(yytext + 1);
}

void symbol()
{
   yylval.str = strdup(yytext);
}

void regex()
{
   yylval.str = malloc(strlen(yytext) - 1);
   strncpy(yylval.str, yytext + 1, strlen(yytext) - 2);
   yylval.str[(strlen(yytext) - 2)] = '\0';
}
