/*
 *  Copyright (C) 2002 Steve Harris, ECS, University of Southampton
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  $Id: tstore_info.c,v 1.9 2004/10/28 07:27:13 theno23 Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <mysql.h>
#include <getopt.h>
#include <locale.h>

#include "rdfsql/rdfsql.h"
#include "rst_parser.h"

int count_thing(RDFSQL *db, const char *desc, const char *col, const char *dist,
	         const char *constr);
void print_res(const char *description, int count);

int main(int argc, char *argv[])
{
    RDFSQL *db;
    char *optstring = RST_OPTSTR "vs";
    int help = 0;
    int verbosity = 0;
    int show_uris = 0;
    int c, opt_index = 0;
    int total, inf;
    rs_result *res;
    rs_triple *t;

    static struct option long_options[] = {
        RST_LONGOPTS,
        { "verbose", 0, 0, 'v' },
        { "show-uris", 0, 0, 's' },
        { 0, 0, 0, 0 }
    };

    setlocale(LC_ALL, NULL);

    while ((c = getopt_long (argc, argv, optstring, long_options, &opt_index)) != -1) {
        if (!rst_handle_option(c, optarg)) {
            if (c == 'v') {
                verbosity++;
            } else if (c == 's') {
                show_uris = 1;
            } else {
                help++;
            }
        }
    }

    if (help) {
        printf("Usage: %s "RST_OPTIONS" [-v] [-s]\n", argv[0]);
        printf(RST_HELP);
        printf("   -v, --verbose   increase verbosity\n");
        printf("   -s, --show-uris list all uris in KB\n");
        printf("This command shows some status about the KB.\n");
        return 1;
    }

    db = rst_connect(NULL);

    count_thing(db, TBLPREFIX "models", TBLPREFIX "models", "*", "");
    total = count_thing(db, TBLPREFIX"triples", TBLPREFIX"triples", "*", "");
    inf = count_thing(db, "   inferred (FC)", TBLPREFIX"triples", "*",
		      " WHERE inferred != 0");
    print_res("   ground facts", total - inf);
    count_thing(db, TBLPREFIX"resources", TBLPREFIX"resources", "*", "");
    count_thing(db, TBLPREFIX"literals", TBLPREFIX"literals", "*", "");
    count_thing(db, "classes", TBLPREFIX"triples t1, "TBLPREFIX"triples t2", "t1.subject", "  WHERE t1.predicate=0xc74e2b735dd8dc85 AND t2.subject=t1.object AND t2.predicate=0xb6c99360a2925e4b AND (t2.object=0xad808933a855b511 OR t2.object=0x6be100d081a43850) AND t2.literal=0");
    count_thing(db, "properties", TBLPREFIX"triples", "subject", " WHERE predicate=-4085280037482734459 AND object=-9222634323742357029");

    if (show_uris) {
	printf("\nURIs:\n");
	res = rs_find_all_resources(db, 0, NULL);
	while ((t = rs_next_triple(res))) {
	    printf("\t%s\n", t->subject);
	}
    }

    return 0;
}

int count_thing(RDFSQL *db, const char *desc, const char *col, const char *dist,
	         const char *constr)
{
    MYSQL_ROW row;
    MYSQL_RES *res;
    int count;
    char tmp[256];

    if (*dist == '*') {
	snprintf(tmp, 255, "SELECT count(*) FROM %s%s", col, constr);
    } else {
	snprintf(tmp, 255, "SELECT count(DISTINCT %s) FROM %s%s", dist, col,
		 constr);
    }
    if (mysql_query(db->db, tmp)) {
	fprintf(stderr, "SQL error in '%s' %d: %s\n", tmp,
	    mysql_errno(db->db), mysql_error(db->db));
	exit(1);
    }
    res = mysql_store_result(db->db);
    row = mysql_fetch_row(res);
    count = atoi(row[0]);
    mysql_free_result(res);
    print_res(desc, count);

    return count;
}

void print_res(const char *description, int count)
{
    static const char *space = "                ";
    printf("%s%s %12d\n", description, space + strlen(description), count);
}

/* vi:set ts=8 sts=4 sw=4: */
